@@ -20,6 +20,7 @@
 */
 
 #include "util/util.h"
+#include "util/child_common.h"
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/time.h>
@@ -51,6 +52,10 @@
 #include "sbus/sssd_dbus.h"
 #include "monitor/monitor_interfaces.h"
 
+#ifdef USE_KEYRING
+#include <keyutils.h>
+#endif
+
 /* ping time cannot be less then once every few seconds or the
  * monitor will get crazy hammering children with messages */
 #define MONITOR_DEF_PING_TIME 10
@@ -61,12 +66,21 @@
 #define KRB5_RCACHE_DIR_DISABLE "__LIBKRB5_DEFAULTS__"
 
 int cmdline_debug_level;
+int cmdline_debug_timestamps;
+int cmdline_debug_microseconds;
 
 struct svc_spy;
 
+enum mt_svc_type {
+    MT_SVC_SERVICE,
+    MT_SVC_PROVIDER
+};
+
 struct mt_svc {
     struct mt_svc *prev;
     struct mt_svc *next;
+    enum mt_svc_type type;
+
     struct sbus_connection *conn;
     struct svc_spy *conn_spy;
 
@@ -85,10 +99,15 @@ struct mt_svc {
     int restarts;
     time_t last_restart;
     int failed_pongs;
+    DBusPendingCall *pending;
 
     int debug_level;
 
     struct tevent_timer *ping_ev;
+
+    struct sss_child_ctx *child_ctx;
+
+    struct tevent_timer *sigkill_ev;
 };
 
 struct config_file_callback {
@@ -125,6 +144,7 @@ struct mt_ctx {
     bool services_started;
     struct netlink_ctx *nlctx;
     const char *conf_path;
+    struct sss_sigchild_ctx *sigchld_ctx;
 };
 
 static int start_service(struct mt_svc *mt_svc);
@@ -135,32 +155,31 @@ static int service_send_ping(struct mt_svc *svc);
 static int service_signal_reset_offline(struct mt_svc *svc);
 static void ping_check(DBusPendingCall *pending, void *data);
 
-static int service_check_alive(struct mt_svc *svc);
-
 static void set_tasks_checker(struct mt_svc *srv);
-static void set_global_checker(struct mt_ctx *ctx);
 static int monitor_kill_service (struct mt_svc *svc);
 
 static int get_service_config(struct mt_ctx *ctx, const char *name,
                               struct mt_svc **svc_cfg);
 static int get_provider_config(struct mt_ctx *ctx, const char *name,
                               struct mt_svc **svc_cfg);
-static int add_new_service(struct mt_ctx *ctx, const char *name);
-static int add_new_provider(struct mt_ctx *ctx, const char *name);
+static int add_new_service(struct mt_ctx *ctx,
+                           const char *name,
+                           int restarts);
+static int add_new_provider(struct mt_ctx *ctx,
+                            const char *name,
+                            int restarts);
 
 static int mark_service_as_started(struct mt_svc *svc);
 
 static int monitor_cleanup(void);
 
-static void network_status_change_cb(enum network_change state,
-                                     void *cb_data)
+static void network_status_change_cb(void *cb_data)
 {
     struct mt_svc *iter;
     struct mt_ctx *ctx = (struct mt_ctx *) cb_data;
 
-    if (state != NL_ROUTE_UP) return;
-
-    DEBUG(9, ("A new route has appeared, signaling providers to reset offline status\n"));
+    DEBUG(SSSDBG_TRACE_INTERNAL, ("A networking status change detected "
+          "signaling providers to reset offline status\n"));
     for (iter = ctx->svc_list; iter; iter = iter->next) {
         /* Don't signal services, only providers */
         if (iter->provider) {
@@ -311,6 +330,11 @@ static int svc_destructor(void *mem)
         DLIST_REMOVE(svc->mt_ctx->svc_list, svc);
     }
 
+    /* Cancel any pending pings */
+    if (svc->pending) {
+        dbus_pending_call_cancel(svc->pending);
+    }
+
     /* svc is beeing freed, neutralize the spy */
     if (svc->conn_spy) {
         talloc_set_destructor((TALLOC_CTX *)svc->conn_spy, NULL);
@@ -386,7 +410,7 @@ static int mark_service_as_started(struct mt_svc *svc)
         DEBUG(4, ("Now starting services!\n"));
         /* then start all services */
         for (i = 0; ctx->services[i]; i++) {
-            add_new_service(ctx, ctx->services[i]);
+            add_new_service(ctx, ctx->services[i], 0);
         }
     }
 
@@ -413,7 +437,7 @@ static void services_startup_timeout(struct tevent_context *ev,
         DEBUG(4, ("Now starting services!\n"));
         /* then start all services */
         for (i = 0; ctx->services[i]; i++) {
-            add_new_service(ctx, ctx->services[i]);
+            add_new_service(ctx, ctx->services[i], 0);
         }
     }
 }
@@ -469,98 +493,37 @@ static int monitor_dbus_init(struct mt_ctx *ctx)
     return ret;
 }
 
-static void svc_try_restart(struct mt_svc *svc, time_t now)
-{
-    int ret;
-
-    DLIST_REMOVE(svc->mt_ctx->svc_list, svc);
-    if (svc->last_restart != 0) {
-        if ((now - svc->last_restart) > 30) { /* TODO: get val from config */
-            /* it was long ago reset restart threshold */
-            svc->restarts = 0;
-        }
-    }
-
-    /* restart the process */
-    if (svc->restarts > 3) { /* TODO: get val from config */
-        DEBUG(0, ("Process [%s], definitely stopped!\n", svc->name));
-        talloc_free(svc);
-        return;
-    }
-
-    /* Shut down the current ping timer so it will restart
-     * cleanly in start_service()
-     */
-    talloc_free(svc->ping_ev);
-
-    ret = start_service(svc);
-    if (ret != EOK) {
-        DEBUG(0,("Failed to restart service '%s'\n", svc->name));
-        talloc_free(svc);
-        return;
-    }
-
-    svc->restarts++;
-    svc->last_restart = now;
-    return;
-}
-
 static void tasks_check_handler(struct tevent_context *ev,
                                 struct tevent_timer *te,
                                 struct timeval t, void *ptr)
 {
     struct mt_svc *svc = talloc_get_type(ptr, struct mt_svc);
-    time_t now = time(NULL);
-    bool process_alive = true;
     int ret;
 
-    ret = service_check_alive(svc);
+    ret = service_send_ping(svc);
     switch (ret) {
     case EOK:
         /* all fine */
         break;
 
-    case ECHILD:
-        DEBUG(1,("Process (%s) is stopped!\n", svc->name));
-        process_alive = false;
+    case ENXIO:
+        DEBUG(1,("Child (%s) not responding! (yet)\n", svc->name));
         break;
 
     default:
-        /* TODO: should we tear down it ? */
-        DEBUG(1,("Checking for service %s(%d) failed!!\n",
-                 svc->name, svc->pid));
+        /* TODO: should we tear it down ? */
+        DEBUG(1,("Sending a message to service (%s) failed!!\n", svc->name));
         break;
     }
 
-    if (process_alive) {
-        ret = service_send_ping(svc);
-        switch (ret) {
-        case EOK:
-            /* all fine */
-            break;
+    if (svc->failed_pongs >= 3) {
+        /* too long since we last heard of this process */
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              ("Killing service [%s], not responding to pings!\n",
+               svc->name));
 
-        case ENXIO:
-            DEBUG(1,("Child (%s) not responding! (yet)\n", svc->name));
-            break;
-
-        default:
-            /* TODO: should we tear it down ? */
-            DEBUG(1,("Sending a message to service (%s) failed!!\n", svc->name));
-            break;
-        }
-
-        if (svc->failed_pongs >= 3) {
-            /* too long since we last heard of this process */
-            DEBUG(1,
-                  ("Killing service [%s], not responding to pings!\n",
-                   svc->name));
-            monitor_kill_service(svc);
-            process_alive = false;
-        }
-    }
-
-    if (!process_alive) {
-        svc_try_restart(svc, now);
+        /* Kill the service. The SIGCHLD handler will restart it */
+        monitor_kill_service(svc);
         return;
     }
 
@@ -585,77 +548,54 @@ static void set_tasks_checker(struct mt_svc *svc)
     svc->ping_ev = te;
 }
 
-static void global_checks_handler(struct tevent_context *ev,
-                                  struct tevent_timer *te,
-                                  struct timeval t, void *ptr)
-{
-    struct mt_ctx *ctx = talloc_get_type(ptr, struct mt_ctx);
-    struct mt_svc *svc;
-    int status;
-    pid_t pid;
-
-    if (!ctx->check_children) {
-        goto done;
-    }
-
-    errno = 0;
-    pid = waitpid(0, &status, WNOHANG);
-    if (pid == 0) {
-        goto done;
-    }
-
-    if (pid == -1) {
-        DEBUG(0, ("waitpid returned -1 (errno:%d[%s])\n",
-                  errno, strerror(errno)));
-        goto done;
-    }
-
-    /* let's see if it is a known service, and try to restart it */
-    for (svc = ctx->svc_list; svc; svc = svc->next) {
-        if (svc->pid == pid) {
-            time_t now = time(NULL);
-            DEBUG(1, ("Service [%s] did exit\n", svc->name));
-            svc_try_restart(svc, now);
-            goto done;
-        }
-    }
-    if (svc == NULL) {
-        DEBUG(0, ("Unknown child (%d) did exit\n", pid));
-    }
-
-done:
-    set_global_checker(ctx);
-}
-
-static void set_global_checker(struct mt_ctx *ctx)
-{
-    struct tevent_timer *te = NULL;
-    struct timeval tv;
-
-    gettimeofday(&tv, NULL);
-    tv.tv_sec += 1; /* once a second */
-    tv.tv_usec = 0;
-    te = tevent_add_timer(ctx->ev, ctx, tv, global_checks_handler, ctx);
-    if (te == NULL) {
-        DEBUG(0, ("failed to add global checker event! PANIC TIME!\n"));
-        /* FIXME: is this right ? shoulkd we try to clean up first ?*/
-        exit(-1);
-    }
-}
-
+static void mt_svc_sigkill(struct tevent_context *ev,
+                           struct tevent_timer *te,
+                           struct timeval t, void *ptr);
 static int monitor_kill_service (struct mt_svc *svc)
 {
     int ret;
+    struct timeval tv;
+
     ret = kill(svc->pid, SIGTERM);
     if (ret != EOK) {
-        DEBUG(0,("Sending signal to child (%s:%d) failed! "
-                 "Ignore and pretend child is dead.\n",
-                 svc->name, svc->pid));
+        DEBUG(SSSDBG_FATAL_FAILURE,
+              ("Sending signal to child (%s:%d) failed! "
+               "Ignore and pretend child is dead.\n",
+               svc->name, svc->pid));
+        talloc_free(svc);
     }
 
+    /* Set up a timer to send SIGKILL if this process
+     * doesn't exit within sixty seconds
+     */
+    tv = tevent_timeval_current_ofs(60, 0);
+    svc->sigkill_ev = tevent_add_timer(svc->mt_ctx->ev, svc, tv,
+                                       mt_svc_sigkill, svc);
+
     return ret;
 }
 
+static void mt_svc_sigkill(struct tevent_context *ev,
+                           struct tevent_timer *te,
+                           struct timeval t, void *ptr)
+{
+    int ret;
+    struct mt_svc *svc = talloc_get_type(ptr, struct mt_svc);
+
+    DEBUG(SSSDBG_FATAL_FAILURE,
+          ("[%s][%d] is not responding to SIGTERM. Sending SIGKILL.\n",
+           svc->name, svc->pid));
+
+    ret = kill(svc->pid, SIGKILL);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE,
+              ("Sending signal to child (%s:%d) failed! "
+              "Ignore and pretend child is dead.\n",
+              svc->name, svc->pid));
+        talloc_free(svc);
+    }
+}
+
 static void reload_reply(DBusPendingCall *pending, void *data)
 {
     DBusMessage *reply;
@@ -814,7 +754,8 @@ static int check_local_domain_unique(struct sss_domain_info *domains)
 
 static char *check_services(char **services)
 {
-    const char *known_services[] = { "nss", "pam", NULL };
+    const char *known_services[] = { "nss", "pam", "sudo", "autofs", "ssh",
+                                     NULL };
     int i;
     int ii;
 
@@ -900,6 +841,7 @@ static int get_service_config(struct mt_ctx *ctx, const char *name,
     int ret;
     char *path;
     struct mt_svc *svc;
+    time_t now = time(NULL);
 
     *svc_cfg = NULL;
 
@@ -908,6 +850,7 @@ static int get_service_config(struct mt_ctx *ctx, const char *name,
         return ENOMEM;
     }
     svc->mt_ctx = ctx;
+    svc->type = MT_SVC_SERVICE;
 
     talloc_set_destructor((TALLOC_CTX *)svc, svc_destructor);
 
@@ -939,34 +882,54 @@ static int get_service_config(struct mt_ctx *ctx, const char *name,
     }
 
     if (!svc->command) {
-        if (cmdline_debug_level == SSS_UNRESOLVED_DEBUG_LEVEL) {
-            svc->command = talloc_asprintf(svc, "%s/sssd_%s %s%s",
-                                           SSSD_LIBEXEC_PATH,
-                                           svc->name,
-                                           debug_timestamps?
-                                                  "": "--debug-timestamps=0 ",
-                                           debug_to_file?
-                                                  "--debug-to-files":"");
-        } else {
-            /* If the debug level was specified at the command-line,
-             * make sure to pass it into the children, overriding the
-             * config file.
-             */
-            svc->command = talloc_asprintf(svc, "%s/sssd_%s -d %d%s%s",
-                                           SSSD_LIBEXEC_PATH,
-                                           svc->name,
-                                           cmdline_debug_level,
-                                           debug_timestamps ?
-                                               "" :
-                                               " --debug-timestamps=0",
-                                           debug_to_file ?
-                                               " --debug-to-files" :
-                                               "");
-        }
+        svc->command = talloc_asprintf(
+            svc, "%s/sssd_%s", SSSD_LIBEXEC_PATH, svc->name
+        );
         if (!svc->command) {
             talloc_free(svc);
             return ENOMEM;
         }
+
+        if (cmdline_debug_level != SSSDBG_UNRESOLVED) {
+            svc->command = talloc_asprintf_append(
+                svc->command, " -d %#.4x", cmdline_debug_level
+            );
+            if (!svc->command) {
+                talloc_free(svc);
+                return ENOMEM;
+            }
+        }
+
+        if (cmdline_debug_timestamps != SSSDBG_TIMESTAMP_UNRESOLVED) {
+            svc->command = talloc_asprintf_append(
+                svc->command, " --debug-timestamps=%d", cmdline_debug_timestamps
+            );
+            if (!svc->command) {
+                talloc_free(svc);
+                return ENOMEM;
+            }
+        }
+
+        if (cmdline_debug_microseconds != SSSDBG_MICROSECONDS_UNRESOLVED) {
+            svc->command = talloc_asprintf_append(
+                svc->command, " --debug-microseconds=%d",
+                cmdline_debug_microseconds
+            );
+            if (!svc->command) {
+                talloc_free(svc);
+                return ENOMEM;
+            }
+        }
+
+        if (debug_to_file) {
+            svc->command = talloc_strdup_append(
+                svc->command, " --debug-to-files"
+            );
+            if (!svc->command) {
+                talloc_free(svc);
+                return ENOMEM;
+            }
+        }
     }
 
     ret = confdb_get_int(ctx->cdb, svc, path,
@@ -983,13 +946,17 @@ static int get_service_config(struct mt_ctx *ctx, const char *name,
         svc->ping_time = MONITOR_DEF_PING_TIME;
     }
 
+    svc->last_restart = now;
+
     *svc_cfg = svc;
     talloc_free(path);
 
     return EOK;
 }
 
-static int add_new_service(struct mt_ctx *ctx, const char *name)
+static int add_new_service(struct mt_ctx *ctx,
+                           const char *name,
+                           int restarts)
 {
     int ret;
     struct mt_svc *svc;
@@ -998,6 +965,7 @@ static int add_new_service(struct mt_ctx *ctx, const char *name)
     if (ret != EOK) {
         return ret;
     }
+    svc->restarts = restarts;
 
     ret = start_service(svc);
     if (ret != EOK) {
@@ -1014,6 +982,7 @@ static int get_provider_config(struct mt_ctx *ctx, const char *name,
     int ret;
     char *path;
     struct mt_svc *svc;
+    time_t now = time(NULL);
 
     *svc_cfg = NULL;
 
@@ -1022,6 +991,7 @@ static int get_provider_config(struct mt_ctx *ctx, const char *name,
         return ENOMEM;
     }
     svc->mt_ctx = ctx;
+    svc->type = MT_SVC_PROVIDER;
 
     talloc_set_destructor((TALLOC_CTX *)svc, svc_destructor);
 
@@ -1085,36 +1055,65 @@ static int get_provider_config(struct mt_ctx *ctx, const char *name,
 
     /* if there are no custom commands, build a default one */
     if (!svc->command) {
-        if (cmdline_debug_level == SSS_UNRESOLVED_DEBUG_LEVEL) {
-            svc->command = talloc_asprintf(svc,
-                                "%s/sssd_be --domain %s%s%s",
-                                SSSD_LIBEXEC_PATH,
-                                svc->name,
-                                debug_timestamps ? ""
-                                                 : " --debug-timestamps=0",
-                                debug_to_file ? " --debug-to-files" : "");
-        } else {
-            svc->command = talloc_asprintf(svc,
-                                "%s/sssd_be --domain %s -d %d%s%s ",
-                                SSSD_LIBEXEC_PATH,
-                                svc->name,
-                                cmdline_debug_level,
-                                debug_timestamps ? ""
-                                                 : " --debug-timestamps=0",
-                                debug_to_file ? " --debug-to-files" : "");
-        }
-
+        svc->command = talloc_asprintf(
+            svc, "%s/sssd_be --domain %s", SSSD_LIBEXEC_PATH, svc->name
+        );
         if (!svc->command) {
             talloc_free(svc);
             return ENOMEM;
         }
+
+        if (cmdline_debug_level != SSSDBG_UNRESOLVED) {
+            svc->command = talloc_asprintf_append(
+                svc->command, " -d %#.4x", cmdline_debug_level
+            );
+            if (!svc->command) {
+                talloc_free(svc);
+                return ENOMEM;
+            }
+        }
+
+        if (cmdline_debug_timestamps != SSSDBG_TIMESTAMP_UNRESOLVED) {
+            svc->command = talloc_asprintf_append(
+                svc->command, " --debug-timestamps=%d", cmdline_debug_timestamps
+            );
+            if (!svc->command) {
+                talloc_free(svc);
+                return ENOMEM;
+            }
+        }
+
+        if (cmdline_debug_microseconds != SSSDBG_MICROSECONDS_UNRESOLVED) {
+            svc->command = talloc_asprintf_append(
+                svc->command, " --debug-microseconds=%d",
+                cmdline_debug_microseconds
+            );
+            if (!svc->command) {
+                talloc_free(svc);
+                return ENOMEM;
+            }
+        }
+
+        if (debug_to_file) {
+            svc->command = talloc_strdup_append(
+                svc->command, " --debug-to-files"
+            );
+            if (!svc->command) {
+                talloc_free(svc);
+                return ENOMEM;
+            }
+        }
     }
 
+    svc->last_restart = now;
+
     *svc_cfg = svc;
     return EOK;
 }
 
-static int add_new_provider(struct mt_ctx *ctx, const char *name)
+static int add_new_provider(struct mt_ctx *ctx,
+                            const char *name,
+                            int restarts)
 {
     int ret;
     struct mt_svc *svc;
@@ -1125,6 +1124,7 @@ static int add_new_provider(struct mt_ctx *ctx, const char *name)
                   name));
         return ret;
     }
+    svc->restarts = restarts;
 
     if (strcasecmp(svc->provider, "local") == 0) {
         /* The LOCAL provider requires no back-end currently
@@ -1331,26 +1331,6 @@ static void signal_offline_reset(struct tevent_context *ev,
     }
 }
 
-int read_config_file(const char *config_file)
-{
-    int ret;
-    struct collection_item *sssd_config = NULL;
-    struct collection_item *error_list = NULL;
-
-    /* Read the configuration into a collection */
-    ret = config_from_file("sssd", config_file, &sssd_config,
-                           INI_STOP_ON_ANY, &error_list);
-    if (ret != EOK) {
-        DEBUG(0, ("Parse error reading configuration file [%s]\n",
-                  config_file));
-        print_file_parsing_errors(stderr, error_list);
-    }
-
-    free_ini_config_errors(error_list);
-    free_ini_config(sssd_config);
-    return ret;
-}
-
 static int monitor_ctx_destructor(void *mem)
 {
     struct mt_ctx *mon = talloc_get_type(mem, struct mt_ctx);
@@ -1965,6 +1945,10 @@ int monitor_process_init(struct mt_ctx *ctx,
         return EIO;
     }
 
+    /* Set up the SIGCHLD handler */
+    ret = sss_sigchld_init(ctx, ctx->ev, &ctx->sigchld_ctx);
+    if (ret != EOK) return ret;
+
 #if 0
     This feature is incomplete and can leave the SSSD in a bad state if the
     config file is changed while the SSSD is running.
@@ -2016,7 +2000,7 @@ int monitor_process_init(struct mt_ctx *ctx,
     /* start providers */
     num_providers = 0;
     for (dom = ctx->domains; dom; dom = dom->next) {
-        ret = add_new_provider(ctx, dom->name);
+        ret = add_new_provider(ctx, dom->name, 0);
         if (ret != EOK && ret != ENOENT) {
             return ret;
         }
@@ -2042,13 +2026,10 @@ int monitor_process_init(struct mt_ctx *ctx,
         /* No providers start services immediately
          * Normally this means only LOCAL is configured */
         for (i = 0; ctx->services[i]; i++) {
-            add_new_service(ctx, ctx->services[i]);
+            add_new_service(ctx, ctx->services[i], 0);
         }
     }
 
-    /* now start checking for global events */
-    set_global_checker(ctx);
-
     return EOK;
 }
 
@@ -2141,7 +2122,7 @@ static int service_send_ping(struct mt_svc *svc)
 
     ret = sbus_conn_send(svc->conn, msg,
                          svc->ping_time * 1000, /* milliseconds */
-                         ping_check, svc, NULL);
+                         ping_check, svc, &svc->pending);
     dbus_message_unref(msg);
     return ret;
 }
@@ -2155,6 +2136,13 @@ static void ping_check(DBusPendingCall *pending, void *data)
     int type;
 
     svc = talloc_get_type(data, struct mt_svc);
+    if (!svc) {
+        /* The connection probably went down before the callback fired.
+         * Not much we can do. */
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Invalid service pointer.\n"));
+        return;
+    }
+    svc->pending = NULL;
 
     reply = dbus_pending_call_steal_reply(pending);
     if (!reply) {
@@ -2193,7 +2181,7 @@ static void ping_check(DBusPendingCall *pending, void *data)
         /* Increase failed pong count */
         if (strnlen(dbus_error_name, len + 1) == len
                 && strncmp(dbus_error_name, DBUS_ERROR_NO_REPLY, len) == 0) {
-            DEBUG(1,
+            DEBUG(SSSDBG_CRIT_FAILURE,
                   ("A service PING timed out on [%s]. "
                    "Attempt [%d]\n",
                    svc->name, svc->failed_pongs));
@@ -2201,7 +2189,7 @@ static void ping_check(DBusPendingCall *pending, void *data)
             break;
         }
 
-        DEBUG(0,
+        DEBUG(SSSDBG_FATAL_FAILURE,
               ("A service PING returned an error [%s], closing connection.\n",
                dbus_error_name));
         /* Falling through to default intentionally*/
@@ -2221,39 +2209,6 @@ done:
     dbus_message_unref(reply);
 }
 
-
-
-/* service_check_alive
- * This function checks if the service child is still alive
- */
-static int service_check_alive(struct mt_svc *svc)
-{
-    int status;
-    pid_t pid;
-
-    DEBUG(4,("Checking service %s(%d) is still alive\n", svc->name, svc->pid));
-
-    pid = waitpid(svc->pid, &status, WNOHANG);
-    if (pid == 0) {
-        return EOK;
-    }
-
-    if (pid != svc->pid) {
-        DEBUG(1, ("bad return (%d) from waitpid() waiting for %d\n",
-                  pid, svc->pid));
-        /* TODO: what do we do now ? */
-        return EINVAL;
-    }
-
-    if (WIFEXITED(status)) { /* children exited on it's own */
-        /* TODO: check configuration to see if it was removed
-         * from the list of process to run */
-        DEBUG(0,("Process [%s] exited\n", svc->name));
-    }
-
-    return ECHILD;
-}
-
 static void service_startup_handler(struct tevent_context *ev,
                                     struct tevent_timer *te,
                                     struct timeval t, void *ptr);
@@ -2287,10 +2242,12 @@ static int start_service(struct mt_svc *svc)
     return EOK;
 }
 
+static void mt_svc_exit_handler(int pid, int wait_status, void *pvt);
 static void service_startup_handler(struct tevent_context *ev,
                                     struct tevent_timer *te,
                                     struct timeval t, void *ptr)
 {
+    errno_t ret;
     struct mt_svc *mt_svc;
     char **args;
 
@@ -2302,13 +2259,30 @@ static void service_startup_handler(struct tevent_context *ev,
     mt_svc->pid = fork();
     if (mt_svc->pid != 0) {
         if (mt_svc->pid == -1) {
-            DEBUG(0, ("Could not fork child to start service [%s]. Continuing.\n", mt_svc->name))
+            DEBUG(0, ("Could not fork child to start service [%s]. "
+                      "Continuing.\n", mt_svc->name));
             return;
         }
 
         /* Parent */
         mt_svc->mt_ctx->check_children = true;
         mt_svc->failed_pongs = 0;
+
+        /* Handle process exit */
+        ret = sss_child_register(mt_svc,
+                                 mt_svc->mt_ctx->sigchld_ctx,
+                                 mt_svc->pid,
+                                 mt_svc_exit_handler,
+                                 mt_svc,
+                                 &mt_svc->child_ctx);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_FATAL_FAILURE,
+                  ("Could not register sigchld handler.\n"));
+            /* Should we exit here? For now, we'll hope this
+             * child never dies, because we can't restart it.
+             */
+        }
+
         DLIST_ADD(mt_svc->mt_ctx->svc_list, mt_svc);
         set_tasks_checker(mt_svc);
 
@@ -2330,12 +2304,65 @@ static void service_startup_handler(struct tevent_context *ev,
     _exit(1);
 }
 
+static void mt_svc_exit_handler(int pid, int wait_status, void *pvt)
+{
+    struct mt_svc *svc = talloc_get_type(pvt, struct mt_svc);
+    time_t now = time(NULL);
+
+    if WIFEXITED(wait_status) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              ("Child [%s] exited with code [%d]\n",
+               svc->name, WEXITSTATUS(wait_status)));
+    } else if WIFSIGNALED(wait_status) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              ("Child [%s] terminated with signal [%d]\n",
+               svc->name, WTERMSIG(wait_status)));
+    } else {
+        DEBUG(0, ("Child [%s] did not exit cleanly\n", svc->name));
+        /* Forcibly kill this child, just in case */
+        kill(svc->pid, SIGKILL);
+
+        /* Return and let us get caught by another
+         * call to the SIGCHLD handler
+         */
+        return;
+    }
+
+    if ((now - svc->last_restart) > 30) { /* TODO: get val from config */
+        svc->restarts = 0;
+    }
+
+    /* Restart the service */
+    if (svc->restarts > 2) { /* TODO: get val from config */
+        DEBUG(SSSDBG_FATAL_FAILURE,
+              ("Process [%s], definitely stopped!\n", svc->name));
+        talloc_free(svc);
+        return;
+    }
+
+    if (svc->type == MT_SVC_SERVICE) {
+        add_new_service(svc->mt_ctx, svc->name, svc->restarts + 1);
+    } else if (svc->type == MT_SVC_PROVIDER) {
+        add_new_provider(svc->mt_ctx, svc->name, svc->restarts + 1);
+    } else {
+        /* Invalid type? */
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              ("BUG: Invalid child process type [%d]\n", svc->type));
+    }
+
+    /* Free the old service (which will also remove it
+     * from the child list)
+     */
+    talloc_free(svc);
+}
+
 int main(int argc, const char *argv[])
 {
     int opt;
     poptContext pc;
     int opt_daemon = 0;
     int opt_interactive = 0;
+    int opt_version = 0;
     char *opt_config_file = NULL;
     char *config_file = NULL;
     int flags = 0;
@@ -2354,9 +2381,14 @@ int main(int argc, const char *argv[])
          _("Run interactive (not a daemon)"), NULL}, \
         {"config", 'c', POPT_ARG_STRING, &opt_config_file, 0, \
          _("Specify a non-default config file"), NULL}, \
+         {"version", '\0', POPT_ARG_NONE, &opt_version, 0, \
+          _("Print version number and exit"), NULL }, \
         POPT_TABLEEND
     };
 
+    /* Set debug level to invalid value so we can deside if -d 0 was used. */
+    debug_level = SSSDBG_INVALID;
+
     pc = poptGetContext(argv[0], argc, argv, long_options, 0);
     while((opt = poptGetNextOpt(pc)) != -1) {
         switch(opt) {
@@ -2368,10 +2400,19 @@ int main(int argc, const char *argv[])
         }
     }
 
-    /* If the level was passed at the command-line, we want
+    CONVERT_AND_SET_DEBUG_LEVEL(debug_level);
+
+    if (opt_version) {
+        puts(VERSION""DISTRO_VERSION""PRERELEASE_VERSION);
+        return EXIT_SUCCESS;
+    }
+
+    /* If the level or timestamps was passed at the command-line, we want
      * to save it and pass it to the children later.
      */
     cmdline_debug_level = debug_level;
+    cmdline_debug_timestamps = debug_timestamps;
+    cmdline_debug_microseconds = debug_microseconds;
 
     if (opt_daemon && opt_interactive) {
         fprintf(stderr, "Option -i|--interactive is not allowed together with -D|--daemon\n");
@@ -2419,6 +2460,29 @@ int main(int argc, const char *argv[])
         }
     }
 
+#ifdef USE_KEYRING
+    /* Do this before all the forks, it sets the session key ring so all
+     * keys are private to the daemon and cannot be read by any other process
+     * tree */
+
+    /* make a new session */
+    ret = keyctl_join_session_keyring(NULL);
+    if (ret == -1) {
+        sss_log(SSS_LOG_ALERT,
+                "Could not create private keyring session. "
+                "If you store password there they may be easily accessible "
+                "to the root user. (%d, %s)", errno, strerror(errno));
+    }
+
+    ret = keyctl_setperm(KEY_SPEC_SESSION_KEYRING, KEY_POS_ALL);
+    if (ret == -1) {
+        sss_log(SSS_LOG_ALERT,
+                "Could not set permissions on private keyring. "
+                "If you store password there they may be easily accessible "
+                "to the root user. (%d, %s)", errno, strerror(errno));
+    }
+#endif
+
     /* Warn if nscd seems to be running */
     ret = check_file(NSCD_SOCKET_PATH, -1, -1, -1, CHECK_SOCK, NULL, false);
     if (ret == EOK) {
