@@ -46,6 +46,7 @@ struct be_svc_data {
     bool run_callbacks;
 
     struct be_svc_callback *callbacks;
+    struct fo_server *first_resolved;
 };
 
 struct be_failover_ctx {
@@ -62,12 +63,19 @@ int be_fo_is_srv_identifier(const char *server)
     return server && strcasecmp(server, BE_SRV_IDENTIFIER) == 0;
 }
 
-static int be_fo_get_options(TALLOC_CTX *mem_ctx, struct be_ctx *ctx,
+static int be_fo_get_options(struct be_ctx *ctx,
                              struct fo_options *opts)
 {
     errno_t ret;
 
-    /* todo get timeout from configuration */
+    ret = confdb_get_int(ctx->cdb, ctx, ctx->conf_path,
+                         CONFDB_DOMAIN_RESOLV_TIMEOUT,
+                         FO_DEFAULT_SVC_TIMEOUT,
+                         &opts->service_resolv_timeout);
+    if (ret != EOK) {
+        return ret;
+    }
+
     opts->retry_timeout = 30;
     opts->srv_retry_timeout = 14400;
 
@@ -83,7 +91,7 @@ static int be_fo_get_options(TALLOC_CTX *mem_ctx, struct be_ctx *ctx,
 int be_init_failover(struct be_ctx *ctx)
 {
     int ret;
-    int fo_timeout;
+    int resolv_timeout;
     struct fo_options fopts;
 
     if (ctx->be_fo != NULL) {
@@ -96,19 +104,20 @@ int be_init_failover(struct be_ctx *ctx)
     }
 
     ret = confdb_get_int(ctx->cdb, ctx, ctx->conf_path,
-                         CONFDB_DOMAIN_RESOLV_TIMEOUT,
-                         RESOLV_DEFAULT_TIMEOUT, &fo_timeout);
+                         CONFDB_DOMAIN_RESOLV_OP_TIMEOUT,
+                         RESOLV_DEFAULT_TIMEOUT,
+                         &resolv_timeout);
     if (ret != EOK) {
         return ret;
     }
 
-    ret = resolv_init(ctx, ctx->ev, fo_timeout, &ctx->be_fo->resolv);
+    ret = resolv_init(ctx, ctx->ev, resolv_timeout, &ctx->be_fo->resolv);
     if (ret != EOK) {
         talloc_zfree(ctx->be_fo);
         return ret;
     }
 
-    ret = be_fo_get_options(ctx->be_fo, ctx, &fopts);
+    ret = be_fo_get_options(ctx, &fopts);
     if (ret != EOK) {
         talloc_zfree(ctx->be_fo);
         return ret;
@@ -240,8 +249,11 @@ int be_fo_service_add_callback(TALLOC_CTX *memctx,
     return EOK;
 }
 
-int be_fo_add_srv_server(struct be_ctx *ctx, const char *service_name,
-                         const char *query_service, enum be_fo_protocol proto,
+int be_fo_add_srv_server(struct be_ctx *ctx,
+                         const char *service_name,
+                         const char *query_service,
+                         const char *default_discovery_domain,
+                         enum be_fo_protocol proto,
                          bool proto_fallback, void *user_data)
 {
     struct be_svc_data *svc;
@@ -256,7 +268,7 @@ int be_fo_add_srv_server(struct be_ctx *ctx, const char *service_name,
 
     ret = confdb_get_string(ctx->cdb, svc, ctx->conf_path,
                             CONFDB_DOMAIN_DNS_DISCOVERY_NAME,
-                            NULL, &domain);
+                            default_discovery_domain, &domain);
     if (ret != EOK) {
         DEBUG(1, ("Failed reading %s from confdb\n",
                   CONFDB_DOMAIN_DNS_DISCOVERY_NAME));
@@ -331,6 +343,7 @@ struct be_resolve_server_state {
     int attempts;
 
     struct fo_server *srv;
+    bool first_try;
 };
 
 static void be_resolve_server_done(struct tevent_req *subreq);
@@ -338,7 +351,8 @@ static void be_resolve_server_done(struct tevent_req *subreq);
 struct tevent_req *be_resolve_server_send(TALLOC_CTX *memctx,
                                           struct tevent_context *ev,
                                           struct be_ctx *ctx,
-                                          const char *service_name)
+                                          const char *service_name,
+                                          bool first_try)
 {
     struct tevent_req *req, *subreq;
     struct be_resolve_server_state *state;
@@ -359,6 +373,7 @@ struct tevent_req *be_resolve_server_send(TALLOC_CTX *memctx,
 
     state->svc = svc;
     state->attempts = 0;
+    state->first_try = first_try;
 
     subreq = fo_resolve_service_send(state, ev,
                                      ctx->be_fo->resolv,
@@ -388,42 +403,43 @@ static void be_resolve_server_done(struct tevent_req *subreq)
     switch (ret) {
     case EOK:
         if (!state->srv) {
-            tevent_req_error(req, EFAULT);
-            return;
+            ret = EFAULT;
+            goto fail;
         }
         break;
 
     case ENOENT:
         /* all servers have been tried and none
          * was found good, go offline */
-        tevent_req_error(req, EIO);
-        return;
+        ret = EIO;
+        goto fail;
 
     default:
         /* mark server as bad and retry */
         if (!state->srv) {
-            tevent_req_error(req, EFAULT);
-            return;
+            ret = EFAULT;
+            goto fail;
         }
-        DEBUG(6, ("Couldn't resolve server (%s), resolver returned (%d)\n",
-                  fo_get_server_str_name(state->srv), ret));
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              ("Couldn't resolve server (%s), resolver returned (%d)\n",
+              fo_get_server_str_name(state->srv), ret));
 
         state->attempts++;
         if (state->attempts >= 10) {
             DEBUG(2, ("Failed to find a server after 10 attempts\n"));
-            tevent_req_error(req, EIO);
-            return;
+            ret = EIO;
+            goto fail;
         }
 
         /* now try next one */
-        DEBUG(6, ("Trying with the next one!\n"));
+        DEBUG(SSSDBG_TRACE_LIBS, ("Trying with the next one!\n"));
         subreq = fo_resolve_service_send(state, state->ev,
                                          state->ctx->be_fo->resolv,
                                          state->ctx->be_fo->fo_ctx,
                                          state->svc->fo_service);
         if (!subreq) {
-            tevent_req_error(req, ENOMEM);
-            return;
+            ret = ENOMEM;
+            goto fail;
         }
         tevent_req_set_callback(subreq, be_resolve_server_done, req);
 
@@ -431,24 +447,34 @@ static void be_resolve_server_done(struct tevent_req *subreq)
     }
 
     /* all fine we got the server */
+    if (state->svc->first_resolved == NULL || state->first_try == true) {
+        DEBUG(SSSDBG_TRACE_LIBS, ("Saving the first resolved server\n"));
+        state->svc->first_resolved = state->srv;
+    } else if (state->svc->first_resolved == state->srv) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              ("The fail over cycled through all available servers\n"));
+        ret = ENOENT;
+        goto fail;
+    }
 
-    if (debug_level >= 4 && fo_get_server_name(state->srv)) {
+    if (DEBUG_IS_SET(SSSDBG_FUNC_DATA) && fo_get_server_name(state->srv)) {
         struct resolv_hostent *srvaddr;
         char ipaddr[128];
         srvaddr = fo_get_server_hostent(state->srv);
         if (!srvaddr) {
-            DEBUG(3, ("FATAL: No hostent available for server (%s)\n",
-                      fo_get_server_str_name(state->srv)));
-            tevent_req_error(req, EFAULT);
-            return;
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  ("FATAL: No hostent available for server (%s)\n",
+                  fo_get_server_str_name(state->srv)));
+            ret = EFAULT;
+            goto fail;
         }
 
         inet_ntop(srvaddr->family, srvaddr->addr_list[0]->ipaddr,
                   ipaddr, 128);
 
-        DEBUG(4, ("Found address for server %s: [%s] TTL %d\n",
-                  fo_get_server_str_name(state->srv), ipaddr,
-                  srvaddr->addr_list[0]->ttl));
+        DEBUG(SSSDBG_FUNC_DATA, ("Found address for server %s: [%s] TTL %d\n",
+              fo_get_server_str_name(state->srv), ipaddr,
+              srvaddr->addr_list[0]->ttl));
     }
 
     srv_status_change = fo_get_server_hostname_last_change(state->srv);
@@ -468,6 +494,12 @@ static void be_resolve_server_done(struct tevent_req *subreq)
     }
 
     tevent_req_done(req);
+    return;
+
+fail:
+    DEBUG(SSSDBG_TRACE_LIBS, ("Server resolution failed: %d\n", ret));
+    state->svc->first_resolved = NULL;
+    tevent_req_error(req, ret);
 }
 
 int be_resolve_server_recv(struct tevent_req *req, struct fo_server **srv)
@@ -514,3 +546,33 @@ void reset_fo(struct be_ctx *be_ctx)
     fo_reset_services(be_ctx->be_fo->fo_ctx);
 }
 
+void be_fo_set_port_status(struct be_ctx *ctx,
+                           struct fo_server *server,
+                           enum port_status status)
+{
+    struct be_svc_data *svc;
+    struct fo_service *fsvc;
+
+    fo_set_port_status(server, status);
+
+    fsvc = fo_get_server_service(server);
+    if (!fsvc) {
+        DEBUG(SSSDBG_OP_FAILURE, ("BUG: No service associated with server\n"));
+        return;
+    }
+
+    DLIST_FOR_EACH(svc, ctx->be_fo->svcs) {
+        if (svc->fo_service == fsvc) break;
+    }
+
+    if (!svc) {
+        DEBUG(SSSDBG_OP_FAILURE, ("BUG: Unknown service\n"));
+        return;
+    }
+
+    if (status == PORT_WORKING) {
+        /* We were successful in connecting to the server. Cycle through all
+         * available servers next time */
+        svc->first_resolved = NULL;
+    }
+}
