@@ -50,7 +50,7 @@ ipa_hbac_save_list(struct sysdb_ctx *sysdb, bool delete_subdir,
             goto done;
         }
 
-        ret = sysdb_delete_recursive(tmp_ctx, sysdb, base_dn, true);
+        ret = sysdb_delete_recursive(sysdb, base_dn, true);
         if (ret != EOK) {
             DEBUG(1, ("sysdb_delete_recursive failed.\n"));
             goto done;
@@ -77,8 +77,7 @@ ipa_hbac_save_list(struct sysdb_ctx *sysdb, bool delete_subdir,
         }
         DEBUG(9, ("Object name: [%s].\n", object_name));
 
-        ret = sysdb_store_custom(tmp_ctx, sysdb, domain, object_name, subdir,
-                                 list[c]);
+        ret = sysdb_store_custom(sysdb, object_name, subdir, list[c]);
         if (ret != EOK) {
             DEBUG(1, ("sysdb_store_custom failed.\n"));
             goto done;
@@ -180,6 +179,15 @@ replace_attribute_name(const char *old_name,
     return EOK;
 }
 
+static errno_t
+create_empty_grouplist(struct hbac_request_element *el)
+{
+    el->groups = talloc_array(el, const char *, 1);
+    if (!el->groups) return ENOMEM;
+
+    el->groups[0] = NULL;
+    return EOK;
+}
 
 /********************************************
  * Functions for handling conversion to the *
@@ -228,7 +236,7 @@ hbac_ctx_to_rules(TALLOC_CTX *mem_ctx,
         if (ret == EPERM) {
             goto done;
         } else if (ret != EOK) {
-            DEBUG(1, ("Could not construct rules\n"))
+            DEBUG(1, ("Could not construct rules\n"));
             goto done;
         }
     }
@@ -300,7 +308,6 @@ hbac_attrs_to_rule(TALLOC_CTX *mem_ctx,
     /* Get the users */
     ret = hbac_user_attrs_to_rule(new_rule,
                                   hbac_ctx_sysdb(hbac_ctx),
-                                  hbac_ctx_be(hbac_ctx)->domain,
                                   new_rule->name,
                                   hbac_ctx->rules[idx],
                                   &new_rule->users);
@@ -313,7 +320,6 @@ hbac_attrs_to_rule(TALLOC_CTX *mem_ctx,
     /* Get the services */
     ret = hbac_service_attrs_to_rule(new_rule,
                                      hbac_ctx_sysdb(hbac_ctx),
-                                     hbac_ctx_be(hbac_ctx)->domain,
                                      new_rule->name,
                                      hbac_ctx->rules[idx],
                                      &new_rule->services);
@@ -326,7 +332,6 @@ hbac_attrs_to_rule(TALLOC_CTX *mem_ctx,
     /* Get the target hosts */
     ret = hbac_thost_attrs_to_rule(new_rule,
                                    hbac_ctx_sysdb(hbac_ctx),
-                                   hbac_ctx_be(hbac_ctx)->domain,
                                    new_rule->name,
                                    hbac_ctx->rules[idx],
                                    &new_rule->targethosts);
@@ -337,11 +342,13 @@ hbac_attrs_to_rule(TALLOC_CTX *mem_ctx,
     }
 
     /* Get the source hosts */
+
     ret = hbac_shost_attrs_to_rule(new_rule,
                                    hbac_ctx_sysdb(hbac_ctx),
-                                   hbac_ctx_be(hbac_ctx)->domain,
                                    new_rule->name,
                                    hbac_ctx->rules[idx],
+                                   dp_opt_get_bool(hbac_ctx->ipa_options,
+                                                   IPA_HBAC_SUPPORT_SRCHOST),
                                    &new_rule->srchosts);
     if (ret != EOK) {
         DEBUG(1, ("Could not parse source hosts for rule [%s]\n",
@@ -397,7 +404,6 @@ done:
 static errno_t
 hbac_eval_user_element(TALLOC_CTX *mem_ctx,
                        struct sysdb_ctx *sysdb,
-                       struct sss_domain_info *domain,
                        const char *username,
                        struct hbac_request_element **user_element);
 
@@ -441,7 +447,7 @@ hbac_ctx_to_eval_request(TALLOC_CTX *mem_ctx,
     eval_req->request_time = time(NULL);
 
     /* Get user the user name and groups */
-    ret = hbac_eval_user_element(eval_req, sysdb, domain,
+    ret = hbac_eval_user_element(eval_req, sysdb,
                                  pd->user, &eval_req->user);
     if (ret != EOK) goto done;
 
@@ -490,7 +496,6 @@ done:
 static errno_t
 hbac_eval_user_element(TALLOC_CTX *mem_ctx,
                        struct sysdb_ctx *sysdb,
-                       struct sss_domain_info *domain,
                        const char *username,
                        struct hbac_request_element **user_element)
 {
@@ -519,8 +524,7 @@ hbac_eval_user_element(TALLOC_CTX *mem_ctx,
      * This will give us the list of both POSIX and
      * non-POSIX groups that this user belongs to.
      */
-    ret = sysdb_search_user_by_name(tmp_ctx, sysdb, domain,
-                                    users->name, attrs, &msg);
+    ret = sysdb_search_user_by_name(tmp_ctx, sysdb, users->name, attrs, &msg);
     if (ret != EOK) {
         DEBUG(1, ("Could not determine user memberships for [%s]\n",
                   users->name));
@@ -530,12 +534,7 @@ hbac_eval_user_element(TALLOC_CTX *mem_ctx,
     el = ldb_msg_find_element(msg, SYSDB_ORIG_MEMBEROF);
     if (el == NULL || el->num_values == 0) {
         DEBUG(7, ("No groups for [%s]\n", users->name));
-        users->groups = talloc_array(users, const char *, 1);
-        if (users->groups == NULL) {
-            ret = ENOMEM;
-            goto done;
-        }
-        users->groups[0] = NULL;
+        ret = create_empty_grouplist(users);
         goto done;
     }
     DEBUG(7, ("[%d] groups for [%s]\n", el->num_values, users->name));
@@ -629,8 +628,7 @@ hbac_eval_service_element(TALLOC_CTX *mem_ctx,
          * This rule will only match the name or
          * a service category of ALL
          */
-        svc->groups = NULL;
-        ret = EOK;
+        ret = create_empty_grouplist(svc);
         goto done;
     } else if (ret != EOK) {
         goto done;
@@ -646,8 +644,7 @@ hbac_eval_service_element(TALLOC_CTX *mem_ctx,
          * This rule will only match the name or
          * a service category of ALL
          */
-        svc->groups = NULL;
-        ret = EOK;
+        ret = create_empty_grouplist(svc);
         goto done;
     }
 
@@ -718,8 +715,7 @@ hbac_eval_host_element(TALLOC_CTX *mem_ctx,
         /* We don't know the host (probably an rhost)
          * So we can't determine it's groups either.
          */
-        host->groups = NULL;
-        ret = EOK;
+        ret = create_empty_grouplist(host);
         goto done;
     }
 
@@ -740,8 +736,7 @@ hbac_eval_host_element(TALLOC_CTX *mem_ctx,
          * This rule will only match the name or
          * a host category of ALL
          */
-        host->groups = NULL;
-        ret =  EOK;
+        ret = create_empty_grouplist(host);
         goto done;
     } else if (ret != EOK) {
         goto done;
@@ -757,8 +752,7 @@ hbac_eval_host_element(TALLOC_CTX *mem_ctx,
          * This rule will only match the name or
          * a host category of ALL
          */
-        host->groups = NULL;
-        ret = EOK;
+        ret = create_empty_grouplist(host);
         goto done;
     }
 
