@@ -25,228 +25,11 @@
 #include "providers/ipa/ipa_hbac_private.h"
 #include "providers/ldap/sdap_async.h"
 
-struct ipa_hbac_host_state {
-    struct tevent_context *ev;
-    struct sysdb_ctx *sysdb;
-    struct sss_domain_info *dom;
-    struct sdap_handle *sh;
-    struct sdap_options *opts;
-    const char *search_base;
-    const char **attrs;
-
-    /* Return values */
-    size_t host_count;
-    struct sysdb_attrs **hosts;
-
-    size_t hostgroup_count;
-    struct sysdb_attrs **hostgroups;
-};
-
-static void
-ipa_hbac_host_info_done(struct tevent_req *subreq);
-
-static void
-ipa_hbac_hostgroup_info_done(struct tevent_req *subreq);
-
-struct tevent_req *
-ipa_hbac_host_info_send(TALLOC_CTX *mem_ctx,
-                        struct tevent_context *ev,
-                        struct sysdb_ctx *sysdb,
-                        struct sss_domain_info *dom,
-                        struct sdap_handle *sh,
-                        struct sdap_options *opts,
-                        const char *search_base)
-{
-    errno_t ret;
-    struct ipa_hbac_host_state *state;
-    struct tevent_req *req;
-    struct tevent_req *subreq;
-    char *host_filter;
-
-    req = tevent_req_create(mem_ctx, &state, struct ipa_hbac_host_state);
-    if (req == NULL) {
-        return NULL;
-    }
-
-    state->ev = ev;
-    state->sysdb = sysdb;
-    state->dom = dom;
-    state->sh = sh;
-    state->opts = opts;
-    state->search_base = search_base;
-
-    host_filter = talloc_asprintf(state, "(objectClass=%s)", IPA_HOST);
-    if (host_filter == NULL) {
-        ret = ENOMEM;
-        goto immediate;
-    }
-
-    state->attrs = talloc_array(state, const char *, 8);
-    if (state->attrs == NULL) {
-        DEBUG(1, ("Failed to allocate host attribute list.\n"));
-        ret = ENOMEM;
-        goto immediate;
-    }
-    state->attrs[0] = "objectClass";
-    state->attrs[1] = IPA_HOST_SERVERHOSTNAME;
-    state->attrs[2] = IPA_HOST_FQDN;
-    state->attrs[3] = IPA_UNIQUE_ID;
-    state->attrs[4] = IPA_MEMBER;
-    state->attrs[5] = IPA_MEMBEROF;
-    state->attrs[6] = IPA_CN;
-    state->attrs[7] = NULL;
-
-    subreq = sdap_get_generic_send(state, ev, opts, sh, search_base,
-                                   LDAP_SCOPE_SUB, host_filter,
-                                   state->attrs, NULL, 0,
-                                   dp_opt_get_int(opts->basic,
-                                                  SDAP_ENUM_SEARCH_TIMEOUT));
-    if (subreq == NULL) {
-        DEBUG(1, ("Error requesting host info\n"));
-        ret = EIO;
-        goto immediate;
-    }
-    tevent_req_set_callback(subreq, ipa_hbac_host_info_done, req);
-
-    return req;
-
-immediate:
-    if (ret == EOK) {
-        tevent_req_done(req);
-    } else {
-        tevent_req_error(req, ret);
-    }
-    tevent_req_post(req, ev);
-    return req;
-}
-
-static void
-ipa_hbac_host_info_done(struct tevent_req *subreq)
-{
-    errno_t ret;
-    struct tevent_req *req =
-            tevent_req_callback_data(subreq, struct tevent_req);
-    struct ipa_hbac_host_state *state =
-            tevent_req_data(req, struct ipa_hbac_host_state);
-    char *hostgroup_filter;
-
-    ret = sdap_get_generic_recv(subreq, state,
-                                &state->host_count,
-                                &state->hosts);
-    talloc_zfree(subreq);
-    if (ret != EOK) {
-        tevent_req_error(req, ret);
-        return;
-    }
-
-    if (state->host_count == 0) {
-        tevent_req_error(req, ENOENT);
-        return;
-    }
-
-    ret = replace_attribute_name(IPA_MEMBEROF, SYSDB_ORIG_MEMBEROF,
-                                 state->host_count,
-                                 state->hosts);
-    if (ret != EOK) {
-        DEBUG(1, ("Could not replace attribute names\n"));
-        tevent_req_error(req, ret);
-        return;
-    }
-
-    hostgroup_filter = talloc_asprintf(state, "(objectClass=%s)",
-                                              IPA_HOSTGROUP);
-    if (hostgroup_filter == NULL) {
-        tevent_req_error(req, ENOMEM);
-        return;
-    }
-
-    /* Look up host groups */
-    subreq = sdap_get_generic_send(state, state->ev, state->opts, state->sh,
-                                   state->search_base, LDAP_SCOPE_SUB,
-                                   hostgroup_filter, state->attrs, NULL, 0,
-                                   dp_opt_get_int(state->opts->basic,
-                                                  SDAP_ENUM_SEARCH_TIMEOUT));
-    if (subreq == NULL) {
-        DEBUG(1, ("Error requesting host info\n"));
-        tevent_req_error(req, EIO);
-        return;
-    }
-    tevent_req_set_callback(subreq, ipa_hbac_hostgroup_info_done, req);
-}
-
-static void
-ipa_hbac_hostgroup_info_done(struct tevent_req *subreq)
-{
-    errno_t ret;
-    struct tevent_req *req =
-            tevent_req_callback_data(subreq, struct tevent_req);
-    struct ipa_hbac_host_state *state =
-            tevent_req_data(req, struct ipa_hbac_host_state);
-
-    ret = sdap_get_generic_recv(subreq, state,
-                                &state->hostgroup_count,
-                                &state->hostgroups);
-    talloc_zfree(subreq);
-    if (ret != EOK) goto done;
-
-    ret = replace_attribute_name(IPA_MEMBER, SYSDB_ORIG_MEMBER,
-                                 state->hostgroup_count,
-                                 state->hostgroups);
-    if (ret != EOK) {
-        DEBUG(1, ("Could not replace attribute names\n"));
-        goto done;
-    }
-
-    ret = replace_attribute_name(IPA_MEMBEROF, SYSDB_ORIG_MEMBEROF,
-                                 state->hostgroup_count,
-                                 state->hostgroups);
-    if (ret != EOK) {
-        DEBUG(1, ("Could not replace attribute names\n"));
-        goto done;
-    }
-
-done:
-    if (ret == EOK) {
-        tevent_req_done(req);
-    } else {
-        DEBUG(3, ("Error [%d][%s]\n", ret, strerror(ret)));
-        tevent_req_error(req, ret);
-    }
-}
-
-errno_t
-ipa_hbac_host_info_recv(struct tevent_req *req,
-                        TALLOC_CTX *mem_ctx,
-                        size_t *host_count,
-                        struct sysdb_attrs ***hosts,
-                        size_t *hostgroup_count,
-                        struct sysdb_attrs ***hostgroups)
-{
-    size_t c;
-    struct ipa_hbac_host_state *state =
-            tevent_req_data(req, struct ipa_hbac_host_state);
-
-    TEVENT_REQ_RETURN_ON_ERROR(req);
-
-    *host_count = state->host_count;
-    *hosts = talloc_steal(mem_ctx, state->hosts);
-    for (c = 0; c < state->host_count; c++) {
-        /* Guarantee the memory heirarchy of the list */
-        talloc_steal(state->hosts, state->hosts[c]);
-    }
-
-    *hostgroup_count = state->hostgroup_count;
-    *hostgroups = talloc_steal(mem_ctx, state->hostgroups);
-
-    return EOK;
-}
-
 /*
  * Functions to convert sysdb_attrs to the hbac_rule format
  */
 static errno_t hbac_host_attrs_to_rule(TALLOC_CTX *mem_ctx,
                                        struct sysdb_ctx *sysdb,
-                                       struct sss_domain_info *domain,
                                        const char *rule_name,
                                        struct sysdb_attrs *rule_attrs,
                                        const char *category_attr,
@@ -331,7 +114,7 @@ static errno_t hbac_host_attrs_to_rule(TALLOC_CTX *mem_ctx,
         }
 
         /* First check if this is a specific host */
-        ret = sysdb_search_custom(tmp_ctx, sysdb, domain, filter,
+        ret = sysdb_search_custom(tmp_ctx, sysdb, filter,
                                   HBAC_HOSTS_SUBDIR, attrs,
                                   &count, &msgs);
         if (ret != EOK && ret != ENOENT) goto done;
@@ -367,7 +150,7 @@ static errno_t hbac_host_attrs_to_rule(TALLOC_CTX *mem_ctx,
             num_hosts++;
         } else { /* ret == ENOENT */
             /* Check if this is a hostgroup */
-            ret = sysdb_search_custom(tmp_ctx, sysdb, domain, filter,
+            ret = sysdb_search_custom(tmp_ctx, sysdb, filter,
                                       HBAC_HOSTGROUPS_SUBDIR, attrs,
                                       &count, &msgs);
             if (ret != EOK && ret != ENOENT) goto done;
@@ -403,8 +186,9 @@ static errno_t hbac_host_attrs_to_rule(TALLOC_CTX *mem_ctx,
                 num_hostgroups++;
             } else { /* ret == ENOENT */
                 /* Neither a host nor a hostgroup? Skip it */
-                DEBUG(1, ("[%s] does not map to either a host or hostgroup. "
-                          "Skipping\n", member_dn));
+                DEBUG(SSSDBG_TRACE_LIBS,
+                      ("[%s] does not map to either a host or hostgroup. "
+                       "Skipping\n", member_dn));
             }
         }
         talloc_zfree(member_dn);
@@ -441,14 +225,13 @@ done:
 errno_t
 hbac_thost_attrs_to_rule(TALLOC_CTX *mem_ctx,
                          struct sysdb_ctx *sysdb,
-                         struct sss_domain_info *domain,
                          const char *rule_name,
                          struct sysdb_attrs *rule_attrs,
                          struct hbac_rule_element **thosts)
 {
     DEBUG(7, ("Processing target hosts for rule [%s]\n", rule_name));
 
-    return hbac_host_attrs_to_rule(mem_ctx, sysdb, domain,
+    return hbac_host_attrs_to_rule(mem_ctx, sysdb,
                                    rule_name, rule_attrs,
                                    IPA_HOST_CATEGORY, IPA_MEMBER_HOST,
                                    NULL, thosts);
@@ -457,21 +240,37 @@ hbac_thost_attrs_to_rule(TALLOC_CTX *mem_ctx,
 errno_t
 hbac_shost_attrs_to_rule(TALLOC_CTX *mem_ctx,
                          struct sysdb_ctx *sysdb,
-                         struct sss_domain_info *domain,
                          const char *rule_name,
                          struct sysdb_attrs *rule_attrs,
+                         bool support_srchost,
                          struct hbac_rule_element **source_hosts)
 {
     errno_t ret;
     size_t host_count;
-    TALLOC_CTX *tmp_ctx = talloc_new(mem_ctx);
+    TALLOC_CTX *tmp_ctx;
     size_t idx;
     struct ldb_message_element *el;
     struct hbac_rule_element *shosts;
 
-    DEBUG(7, ("Processing source hosts for rule [%s]\n", rule_name));
+    tmp_ctx = talloc_new(mem_ctx);
+    if (tmp_ctx == NULL) return ENOMEM;
 
-    ret = hbac_host_attrs_to_rule(tmp_ctx, sysdb, domain,
+    DEBUG(SSSDBG_TRACE_FUNC, ("Processing source hosts for rule [%s]\n", rule_name));
+
+    if (!support_srchost) {
+        DEBUG(SSSDBG_TRACE_INTERNAL, ("Source hosts disabled, setting ALL\n"));
+        shosts = talloc_zero(tmp_ctx, struct hbac_rule_element);
+        if (shosts == NULL) {
+            ret = ENOMEM;
+            goto done;
+        }
+
+        shosts->category = HBAC_CATEGORY_ALL;
+        ret = EOK;
+        goto done;
+    }
+
+    ret = hbac_host_attrs_to_rule(tmp_ctx, sysdb,
                                   rule_name, rule_attrs,
                                   IPA_SOURCE_HOST_CATEGORY, IPA_SOURCE_HOST,
                                   &host_count, &shosts);
