@@ -23,6 +23,8 @@
 */
 
 
+#include <ctype.h>
+
 #include "providers/ipa/ipa_common.h"
 
 int domain_to_basedn(TALLOC_CTX *memctx, const char *domain, char **basedn)
@@ -52,6 +54,10 @@ int domain_to_basedn(TALLOC_CTX *memctx, const char *domain, char **basedn)
         return ENOMEM;
     }
 
+    for (p=dn; *p; ++p) {
+        *p = tolower(*p);
+    }
+
     *basedn = dn;
     return EOK;
 }
