@@ -30,7 +30,7 @@
 
 #include "util/sss_krb5.h"
 #include "providers/dp_backend.h"
-#include "providers/child_common.h"
+#include "util/child_common.h"
 #include "providers/krb5/krb5_common.h"
 
 #define CCACHE_ENV_NAME "KRB5CCNAME"
