@@ -23,7 +23,7 @@
 */
 
 #include "util/util.h"
-#include "providers/child_common.h"
+#include "util/child_common.h"
 #include "providers/krb5/krb5_common.h"
 #include "providers/krb5/krb5_auth.h"
 #include "src/providers/krb5/krb5_utils.h"
