@@ -516,7 +516,7 @@ int krb5_service_init(TALLOC_CTX *memctx, struct be_ctx *ctx,
         }
 
         if (be_fo_is_srv_identifier(server_spec)) {
-            ret = be_fo_add_srv_server(ctx, service_name, service_name,
+            ret = be_fo_add_srv_server(ctx, service_name, service_name, NULL,
                                        BE_FO_PROTO_UDP, true, NULL);
             if (ret) {
                 DEBUG(0, ("Failed to add server\n"));
