@@ -68,9 +68,16 @@ struct sdap_auth_ctx {
     struct sdap_service *chpass_service;
 };
 
+int sssm_ldap_id_init(struct be_ctx *bectx,
+                      struct bet_ops **ops,
+                      void **pvt_data);
+
 void sdap_check_online(struct be_req *breq);
+void sdap_do_online_check(struct be_req *be_req, struct sdap_id_ctx *ctx);
+
 /* id */
 void sdap_account_info_handler(struct be_req *breq);
+void sdap_handle_account_info(struct be_req *breq, struct sdap_id_ctx *ctx);
 int sdap_id_setup_tasks(struct sdap_id_ctx *ctx);
 
 /* auth */
@@ -82,7 +89,8 @@ void sdap_pam_chpass_handler(struct be_req *breq);
 /* access */
 void sdap_pam_access_handler(struct be_req *breq);
 
-
+/* autofs */
+void sdap_autofs_handler(struct be_req *breq);
 
 void sdap_handler_done(struct be_req *req, int dp_err,
                        int error, const char *errstr);
@@ -114,6 +122,16 @@ int ldap_get_options(TALLOC_CTX *memctx,
                      const char *conf_path,
                      struct sdap_options **_opts);
 
+int ldap_get_sudo_options(TALLOC_CTX *memctx,
+                          struct confdb_ctx *cdb,
+                          const char *conf_path,
+                          struct sdap_options *opts);
+
+int ldap_get_autofs_options(TALLOC_CTX *memctx,
+                            struct confdb_ctx *cdb,
+                            const char *conf_path,
+                            struct sdap_options *opts);
+
 int ldap_id_enumerate_set_timer(struct sdap_id_ctx *ctx, struct timeval tv);
 int ldap_id_cleanup_set_timer(struct sdap_id_ctx *ctx, struct timeval tv);
 
@@ -136,11 +154,23 @@ struct tevent_req *groups_get_send(TALLOC_CTX *memctx,
 int groups_get_recv(struct tevent_req *req, int *dp_error_out);
 
 
-struct tevent_req *netgroup_get_send(TALLOC_CTX *memctx,
-                                                      struct tevent_context *ev,
-                                                      struct sdap_id_ctx *ctx,
-                                                      const char *name);
-int netgroup_get_recv(struct tevent_req *req, int *dp_error_out);
+struct tevent_req *ldap_netgroup_get_send(TALLOC_CTX *memctx,
+                                          struct tevent_context *ev,
+                                          struct sdap_id_ctx *ctx,
+                                          const char *name);
+int ldap_netgroup_get_recv(struct tevent_req *req, int *dp_error_out);
+
+struct tevent_req *
+services_get_send(TALLOC_CTX *mem_ctx,
+                  struct tevent_context *ev,
+                  struct sdap_id_ctx *id_ctx,
+                  const char *name,
+                  const char *protocol,
+                  int filter_type);
+
+errno_t
+services_get_recv(struct tevent_req *req, int *dp_error_out);
+
 /* setup child logging */
 int setup_child(struct sdap_id_ctx *ctx);
 
@@ -156,18 +186,26 @@ errno_t get_sysdb_attr_name(TALLOC_CTX *mem_ctx,
 errno_t list_missing_attrs(TALLOC_CTX *mem_ctx,
                            struct sdap_attr_map *map,
                            size_t map_size,
-                           const char **expected_attrs,
                            struct sysdb_attrs *recvd_attrs,
                            char ***missing_attrs);
 
 bool sdap_is_secure_uri(const char *uri);
 
 char *sdap_get_id_specific_filter(TALLOC_CTX *mem_ctx,
-                                  char *base_filter,
-                                  char *extra_filter);
+                                  const char *base_filter,
+                                  const char *extra_filter);
 
 errno_t msgs2attrs_array(TALLOC_CTX *mem_ctx, size_t count,
                          struct ldb_message **msgs,
                          struct sysdb_attrs ***attrs);
 
+errno_t sdap_parse_search_base(TALLOC_CTX *mem_ctx,
+                               struct dp_option *opts, int class,
+                               struct sdap_search_base ***_search_bases);
+errno_t common_parse_search_base(TALLOC_CTX *mem_ctx,
+                                 const char *unparsed_base,
+                                 const char *class_name,
+                                 const char *old_filter,
+                                 struct sdap_search_base ***_search_bases);
+
 #endif /* _LDAP_COMMON_H_ */
