@@ -65,7 +65,6 @@ struct tevent_req *users_get_send(TALLOC_CTX *memctx,
     struct users_get_state *state;
     const char *attr_name;
     char *clean_name;
-    char *base_filter;
     int ret;
 
     req = tevent_req_create(memctx, &state, struct users_get_state);
@@ -104,21 +103,12 @@ struct tevent_req *users_get_send(TALLOC_CTX *memctx,
         goto fail;
     }
 
-    base_filter = talloc_asprintf(state, "(&(%s=%s)(objectclass=%s))",
-                                  attr_name, clean_name,
-                                  ctx->opts->user_map[SDAP_OC_USER].name);
+    state->filter = talloc_asprintf(state, "(&(%s=%s)(objectclass=%s))",
+                                    attr_name, clean_name,
+                                    ctx->opts->user_map[SDAP_OC_USER].name);
     talloc_zfree(clean_name);
-    if (!base_filter) {
-        DEBUG(2, ("Failed to build the base filter\n"));
-        ret = ENOMEM;
-        goto fail;
-    }
-
-    state->filter = sdap_get_id_specific_filter(state, base_filter,
-           dp_opt_get_string(ctx->opts->basic, SDAP_USER_SEARCH_FILTER));
-    talloc_zfree(base_filter);
     if (!state->filter) {
-        DEBUG(2, ("Failed to build user filter\n"));
+        DEBUG(2, ("Failed to build the base filter\n"));
         ret = ENOMEM;
         goto fail;
     }
@@ -178,10 +168,12 @@ static void users_get_connect_done(struct tevent_req *subreq)
     subreq = sdap_get_users_send(state, state->ev,
                                  state->domain, state->sysdb,
                                  state->ctx->opts,
+                                 state->ctx->opts->user_search_bases,
                                  sdap_id_op_handle(state->op),
                                  state->attrs, state->filter,
                                  dp_opt_get_int(state->ctx->opts->basic,
-                                                SDAP_SEARCH_TIMEOUT));
+                                                SDAP_SEARCH_TIMEOUT),
+                                 false);
     if (!subreq) {
         tevent_req_error(req, ENOMEM);
         return;
@@ -227,8 +219,7 @@ static void users_get_done(struct tevent_req *subreq)
             tevent_req_error(req, ret);
             return;
         case BE_FILTER_NAME:
-            ret = sysdb_delete_user(state, state->sysdb,
-                                    state->domain, state->name, 0);
+            ret = sysdb_delete_user(state->sysdb, state->name, 0);
             if (ret != EOK && ret != ENOENT) {
                 tevent_req_error(req, ret);
                 return;
@@ -242,8 +233,7 @@ static void users_get_done(struct tevent_req *subreq)
                 return;
             }
 
-            ret = sysdb_delete_user(state, state->sysdb,
-                                    state->domain, NULL, uid);
+            ret = sysdb_delete_user(state->sysdb, NULL, uid);
             if (ret != EOK && ret != ENOENT) {
                 tevent_req_error(req, ret);
                 return;
@@ -307,7 +297,6 @@ struct tevent_req *groups_get_send(TALLOC_CTX *memctx,
     struct groups_get_state *state;
     const char *attr_name;
     char *clean_name;
-    char *base_filter;
     int ret;
 
     req = tevent_req_create(memctx, &state, struct groups_get_state);
@@ -346,25 +335,16 @@ struct tevent_req *groups_get_send(TALLOC_CTX *memctx,
         goto fail;
     }
 
-    base_filter = talloc_asprintf(state,
-                                "(&(%s=%s)(objectclass=%s)(%s=*)(&(%s=*)(!(%s=0))))",
-                                attr_name, clean_name,
-                                ctx->opts->group_map[SDAP_OC_GROUP].name,
-                                ctx->opts->group_map[SDAP_AT_GROUP_NAME].name,
-                                ctx->opts->group_map[SDAP_AT_GROUP_GID].name,
-                                ctx->opts->group_map[SDAP_AT_GROUP_GID].name);
+    state->filter = talloc_asprintf(state,
+                                    "(&(%s=%s)(objectclass=%s)(%s=*)(&(%s=*)(!(%s=0))))",
+                                    attr_name, clean_name,
+                                    ctx->opts->group_map[SDAP_OC_GROUP].name,
+                                    ctx->opts->group_map[SDAP_AT_GROUP_NAME].name,
+                                    ctx->opts->group_map[SDAP_AT_GROUP_GID].name,
+                                    ctx->opts->group_map[SDAP_AT_GROUP_GID].name);
     talloc_zfree(clean_name);
-    if (!base_filter) {
-        DEBUG(2, ("Failed to build filter\n"));
-        ret = ENOMEM;
-        goto fail;
-    }
-
-    state->filter = sdap_get_id_specific_filter(state, base_filter,
-           dp_opt_get_string(ctx->opts->basic, SDAP_GROUP_SEARCH_FILTER));
-    talloc_zfree(base_filter);
     if (!state->filter) {
-        DEBUG(2, ("Failed to build group-specific filter\n"));
+        DEBUG(2, ("Failed to build filter\n"));
         ret = ENOMEM;
         goto fail;
     }
@@ -423,10 +403,13 @@ static void groups_get_connect_done(struct tevent_req *subreq)
 
     subreq = sdap_get_groups_send(state, state->ev,
                                   state->domain, state->sysdb,
-                                  state->ctx->opts, sdap_id_op_handle(state->op),
+                                  state->ctx->opts,
+                                  state->ctx->opts->group_search_bases,
+                                  sdap_id_op_handle(state->op),
                                   state->attrs, state->filter,
                                   dp_opt_get_int(state->ctx->opts->basic,
-                                                 SDAP_SEARCH_TIMEOUT));
+                                                 SDAP_SEARCH_TIMEOUT),
+                                  false);
     if (!subreq) {
         tevent_req_error(req, ENOMEM);
         return;
@@ -472,8 +455,7 @@ static void groups_get_done(struct tevent_req *subreq)
             tevent_req_error(req, ret);
             return;
         case BE_FILTER_NAME:
-            ret = sysdb_delete_group(state, state->sysdb,
-                                     state->domain, state->name, 0);
+            ret = sysdb_delete_group(state->sysdb, state->name, 0);
             if (ret != EOK && ret != ENOENT) {
                 tevent_req_error(req, ret);
                 return;
@@ -487,8 +469,7 @@ static void groups_get_done(struct tevent_req *subreq)
                 return;
             }
 
-            ret = sysdb_delete_group(state, state->sysdb,
-                                     state->domain, NULL, gid);
+            ret = sysdb_delete_group(state->sysdb, NULL, gid);
             if (ret != EOK && ret != ENOENT) {
                 tevent_req_error(req, ret);
                 return;
@@ -656,10 +637,7 @@ static void groups_by_user_done(struct tevent_req *subreq)
     }
 
     if (ret == ENOENT) {
-        ret = sysdb_delete_user(state,
-                                state->ctx->be->sysdb,
-                                state->ctx->be->domain,
-                                state->name, 0);
+        ret = sysdb_delete_user(state->ctx->be->sysdb, state->name, 0);
         if (ret != EOK && ret != ENOENT) {
             tevent_req_error(req, ret);
             return;
@@ -688,35 +666,57 @@ static void sdap_check_online_done(struct tevent_req *req);
 void sdap_check_online(struct be_req *be_req)
 {
     struct sdap_id_ctx *ctx;
-    struct tevent_req *req;
 
     ctx = talloc_get_type(be_req->be_ctx->bet_info[BET_ID].pvt_bet_data,
                           struct sdap_id_ctx);
 
+    return sdap_do_online_check(be_req, ctx);
+}
+
+struct sdap_online_check_ctx {
+    struct be_req *be_req;
+    struct sdap_id_ctx *id_ctx;
+};
+
+void sdap_do_online_check(struct be_req *be_req, struct sdap_id_ctx *ctx)
+{
+    struct tevent_req *req;
+    struct sdap_online_check_ctx *check_ctx;
+    errno_t ret;
+
+    check_ctx = talloc_zero(be_req, struct sdap_online_check_ctx);
+    if (!check_ctx) {
+        ret = ENOMEM;
+        DEBUG(SSSDBG_CRIT_FAILURE, ("talloc_zero failed\n"));
+        goto fail;
+    }
+    check_ctx->id_ctx = ctx;
+    check_ctx->be_req = be_req;
+
     req = sdap_cli_connect_send(be_req, be_req->be_ctx->ev, ctx->opts,
-                                be_req->be_ctx, ctx->service, false);
+                                be_req->be_ctx, ctx->service, false,
+                                CON_TLS_DFL, false);
     if (req == NULL) {
         DEBUG(1, ("sdap_cli_connect_send failed.\n"));
-        goto done;
+        ret = EIO;
+        goto fail;
     }
-    tevent_req_set_callback(req, sdap_check_online_done, be_req);
+    tevent_req_set_callback(req, sdap_check_online_done, check_ctx);
 
     return;
-done:
-    sdap_handler_done(be_req, DP_ERR_FATAL, 0, NULL);
+fail:
+    sdap_handler_done(be_req, DP_ERR_FATAL, ret, NULL);
 }
 
 static void sdap_check_online_done(struct tevent_req *req)
 {
-    struct be_req *be_req = tevent_req_callback_data(req, struct be_req);
+    struct sdap_online_check_ctx *check_ctx = tevent_req_callback_data(req,
+                                        struct sdap_online_check_ctx);
     int ret;
     int dp_err = DP_ERR_FATAL;
     bool can_retry;
-    struct sdap_id_ctx *ctx;
     struct sdap_server_opts *srv_opts;
-
-    ctx = talloc_get_type(be_req->be_ctx->bet_info[BET_ID].pvt_bet_data,
-                          struct sdap_id_ctx);
+    struct be_req *be_req;
 
     ret = sdap_cli_connect_recv(req, NULL, &can_retry, NULL, &srv_opts);
     talloc_zfree(req);
@@ -728,20 +728,24 @@ static void sdap_check_online_done(struct tevent_req *req)
     } else {
         dp_err = DP_ERR_OK;
 
-        if (!ctx->srv_opts) {
+        if (!check_ctx->id_ctx->srv_opts) {
             srv_opts->max_user_value = 0;
             srv_opts->max_group_value = 0;
-        } else if (strcmp(srv_opts->server_id, ctx->srv_opts->server_id) == 0
+            srv_opts->max_service_value = 0;
+        } else if (strcmp(srv_opts->server_id, check_ctx->id_ctx->srv_opts->server_id) == 0
                    && srv_opts->supports_usn
-                   && ctx->srv_opts->last_usn > srv_opts->last_usn) {
-            ctx->srv_opts->max_user_value = 0;
-            ctx->srv_opts->max_group_value = 0;
-            ctx->srv_opts->last_usn = srv_opts->last_usn;
+                   && check_ctx->id_ctx->srv_opts->last_usn > srv_opts->last_usn) {
+            check_ctx->id_ctx->srv_opts->max_user_value = 0;
+            check_ctx->id_ctx->srv_opts->max_group_value = 0;
+            check_ctx->id_ctx->srv_opts->max_service_value = 0;
+            check_ctx->id_ctx->srv_opts->last_usn = srv_opts->last_usn;
         }
 
-        sdap_steal_server_opts(ctx, &srv_opts);
+        sdap_steal_server_opts(check_ctx->id_ctx, &srv_opts);
     }
 
+    be_req = check_ctx->be_req;
+    talloc_free(check_ctx);
     sdap_handler_done(be_req, dp_err, 0, NULL);
 }
 
@@ -754,17 +758,29 @@ static void sdap_account_info_users_done(struct tevent_req *req);
 static void sdap_account_info_groups_done(struct tevent_req *req);
 static void sdap_account_info_initgr_done(struct tevent_req *req);
 static void sdap_account_info_netgroups_done(struct tevent_req *req);
+static void sdap_account_info_services_done(struct tevent_req *req);
+void sdap_handle_account_info(struct be_req *breq, struct sdap_id_ctx *ctx);
 
 void sdap_account_info_handler(struct be_req *breq)
 {
     struct sdap_id_ctx *ctx;
+
+    ctx = talloc_get_type(breq->be_ctx->bet_info[BET_ID].pvt_bet_data, struct sdap_id_ctx);
+    if (!ctx) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Could not get sdap ctx\n"));
+        return sdap_handler_done(breq, DP_ERR_FATAL,
+                                 EINVAL, "Invalid request data\n");
+    }
+    return sdap_handle_account_info(breq, ctx);
+}
+
+void sdap_handle_account_info(struct be_req *breq, struct sdap_id_ctx *ctx)
+{
     struct be_acct_req *ar;
     struct tevent_req *req;
     const char *err = "Unknown Error";
     int ret = EOK;
 
-    ctx = talloc_get_type(breq->be_ctx->bet_info[BET_ID].pvt_bet_data, struct sdap_id_ctx);
-
     if (be_is_offline(ctx->be)) {
         return sdap_handler_done(breq, DP_ERR_OFFLINE, EAGAIN, "Offline");
     }
@@ -837,7 +853,7 @@ void sdap_account_info_handler(struct be_req *breq)
             break;
         }
 
-        req = netgroup_get_send(breq, breq->be_ctx->ev, ctx, ar->filter_value);
+        req = ldap_netgroup_get_send(breq, breq->be_ctx->ev, ctx, ar->filter_value);
         if (!req) {
             return sdap_handler_done(breq, DP_ERR_FATAL, ENOMEM, "Out of memory");
         }
@@ -845,6 +861,24 @@ void sdap_account_info_handler(struct be_req *breq)
         tevent_req_set_callback(req, sdap_account_info_netgroups_done, breq);
         break;
 
+    case BE_REQ_SERVICES:
+        /* skip enumerations on demand */
+        if (ar->filter_type == BE_FILTER_ENUM) {
+            return sdap_handler_done(breq, DP_ERR_OK, EOK, "Success");
+        }
+
+        req = services_get_send(breq, breq->be_ctx->ev, ctx,
+                                ar->filter_value,
+                                ar->extra_value,
+                                ar->filter_type);
+        if (!req) {
+            return sdap_handler_done(breq, DP_ERR_FATAL,
+                                     ENOMEM, "Out of memory");
+        }
+        tevent_req_set_callback(req, sdap_account_info_services_done, breq);
+
+        break;
+
     default: /*fail*/
         ret = EINVAL;
         err = "Invalid request type";
@@ -915,8 +949,19 @@ static void sdap_account_info_netgroups_done(struct tevent_req *req)
     struct be_req *breq = tevent_req_callback_data(req, struct be_req);
     int ret, dp_error;
 
-    ret = netgroup_get_recv(req, &dp_error);
+    ret = ldap_netgroup_get_recv(req, &dp_error);
     talloc_zfree(req);
 
     sdap_account_info_complete(breq, dp_error, ret, "Netgroup lookup failed");
 }
+
+static void sdap_account_info_services_done(struct tevent_req *req)
+{
+    struct be_req *breq = tevent_req_callback_data(req, struct be_req);
+    int ret, dp_error;
+
+    ret = services_get_recv(req, &dp_error);
+    talloc_zfree(req);
+
+    sdap_account_info_complete(breq, dp_error, ret, "Service lookup failed");
+}
