@@ -22,10 +22,12 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "providers/child_common.h"
+#include "util/child_common.h"
 #include "providers/ldap/ldap_common.h"
 #include "providers/ldap/sdap_async_private.h"
 #include "providers/ldap/sdap_access.h"
+#include "providers/ldap/sdap_sudo.h"
+#include "providers/ldap/sdap_autofs.h"
 
 static void sdap_shutdown(struct be_req *req);
 
@@ -55,7 +57,7 @@ struct bet_ops sdap_access_ops = {
 };
 
 /* Please use this only for short lists */
-errno_t check_order_list_for_duplicates(char **list, size_t len,
+errno_t check_order_list_for_duplicates(char **list,
                                         bool case_sensitive)
 {
     size_t c;
@@ -285,20 +287,20 @@ int sssm_ldap_access_init(struct be_ctx *bectx,
         order = "filter";
     }
 
-    ret = split_on_separator(access_ctx, order, ',', true, &order_list,
-                             &order_list_len);
+    ret = split_on_separator(access_ctx, order, ',', true,
+                             &order_list, &order_list_len);
     if (ret != EOK) {
         DEBUG(1, ("split_on_separator failed.\n"));
         goto done;
     }
 
-    ret = check_order_list_for_duplicates(order_list, order_list_len, false);
+    ret = check_order_list_for_duplicates(order_list, false);
     if (ret != EOK) {
         DEBUG(1, ("check_order_list_for_duplicates failed.\n"));
         goto done;
     }
 
-    if (order_list_len -1 > LDAP_ACCESS_LAST) {
+    if (order_list_len > LDAP_ACCESS_LAST) {
         DEBUG(1, ("Currently only [%d] different access rules are supported.\n"));
         ret = EINVAL;
         goto done;
@@ -386,6 +388,66 @@ done:
     return ret;
 }
 
+int sssm_ldap_sudo_init(struct be_ctx *be_ctx,
+                        struct bet_ops **ops,
+                        void **pvt_data)
+{
+#ifdef BUILD_SUDO
+    struct sdap_id_ctx *id_ctx;
+    void *data;
+    int ret;
+
+    ret = sssm_ldap_id_init(be_ctx, ops, &data);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Cannot init LDAP ID provider [%d]: %s\n",
+                                    ret, strerror(ret)));
+        return ret;
+    }
+
+    id_ctx = talloc_get_type(data, struct sdap_id_ctx);
+    if (!id_ctx) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("No ID provider?\n"));
+        return EIO;
+    }
+
+    return sdap_sudo_init(be_ctx, id_ctx, ops, pvt_data);
+#else
+    DEBUG(SSSDBG_MINOR_FAILURE, ("Sudo init handler called but SSSD is "
+                                 "built without sudo support, ignoring\n"));
+    return EOK;
+#endif
+}
+
+int sssm_ldap_autofs_init(struct be_ctx *be_ctx,
+                          struct bet_ops **ops,
+                          void **pvt_data)
+{
+#ifdef BUILD_AUTOFS
+    struct sdap_id_ctx *id_ctx;
+    void *data;
+    int ret;
+
+    ret = sssm_ldap_id_init(be_ctx, ops, &data);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Cannot init LDAP ID provider [%d]: %s\n",
+                                    ret, strerror(ret)));
+        return ret;
+    }
+
+    id_ctx = talloc_get_type(data, struct sdap_id_ctx);
+    if (!id_ctx) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("No ID provider?\n"));
+        return EIO;
+    }
+
+    return sdap_autofs_init(be_ctx, id_ctx, ops, pvt_data);
+#else
+    DEBUG(SSSDBG_MINOR_FAILURE, ("Autofs init handler called but SSSD is "
+                                 "built without autofs support, ignoring\n"));
+    return EOK;
+#endif
+}
+
 static void sdap_shutdown(struct be_req *req)
 {
     /* TODO: Clean up any internal data */
