@@ -28,6 +28,7 @@
 #include <tevent.h>
 #include "providers/dp_backend.h"
 #include "providers/ldap/sdap.h"
+#include "providers/ldap/sdap_id_op.h"
 #include "providers/fail_over.h"
 
 struct tevent_req *sdap_connect_send(TALLOC_CTX *memctx,
@@ -45,10 +46,12 @@ struct tevent_req *sdap_get_users_send(TALLOC_CTX *memctx,
                                        struct sss_domain_info *dom,
                                        struct sysdb_ctx *sysdb,
                                        struct sdap_options *opts,
+                                       struct sdap_search_base **search_bases,
                                        struct sdap_handle *sh,
                                        const char **attrs,
-                                       const char *wildcard,
-                                       int timeout);
+                                       const char *filter,
+                                       int timeout,
+                                       bool enumeration);
 int sdap_get_users_recv(struct tevent_req *req,
                         TALLOC_CTX *mem_ctx, char **timestamp);
 
@@ -57,10 +60,12 @@ struct tevent_req *sdap_get_groups_send(TALLOC_CTX *memctx,
                                        struct sss_domain_info *dom,
                                        struct sysdb_ctx *sysdb,
                                        struct sdap_options *opts,
+                                       struct sdap_search_base **search_bases,
                                        struct sdap_handle *sh,
                                        const char **attrs,
-                                       const char *wildcard,
-                                       int timeout);
+                                       const char *filter,
+                                       int timeout,
+                                       bool enumeration);
 int sdap_get_groups_recv(struct tevent_req *req,
                          TALLOC_CTX *mem_ctx, char **timestamp);
 
@@ -69,9 +74,10 @@ struct tevent_req *sdap_get_netgroups_send(TALLOC_CTX *memctx,
                                            struct sss_domain_info *dom,
                                            struct sysdb_ctx *sysdb,
                                            struct sdap_options *opts,
+                                           struct sdap_search_base **search_bases,
                                            struct sdap_handle *sh,
                                            const char **attrs,
-                                           const char *wildcard,
+                                           const char *filter,
                                            int timeout);
 int sdap_get_netgroups_recv(struct tevent_req *req,
                             TALLOC_CTX *mem_ctx, char **timestamp,
@@ -87,6 +93,7 @@ struct tevent_req *sdap_kinit_send(TALLOC_CTX *memctx,
                                    const char *keytab,
                                    const char *principal,
                                    const char *realm,
+                                   bool canonicalize,
                                    int lifetime);
 
 int sdap_kinit_recv(struct tevent_req *req,
@@ -125,12 +132,29 @@ int sdap_exop_modify_passwd_recv(struct tevent_req *req, TALLOC_CTX *mem_ctx,
                                  enum sdap_result *result,
                                  char **user_error_msg);
 
+struct tevent_req *
+sdap_modify_shadow_lastchange_send(TALLOC_CTX *mem_ctx,
+                             struct tevent_context *ev,
+                             struct sdap_handle *sh,
+                             const char *dn,
+                             char *lastchanged_name);
+
+errno_t sdap_modify_shadow_lastchange_recv(struct tevent_req *req);
+
+enum connect_tls {
+    CON_TLS_DFL,
+    CON_TLS_ON,
+    CON_TLS_OFF
+};
+
 struct tevent_req *sdap_cli_connect_send(TALLOC_CTX *memctx,
                                          struct tevent_context *ev,
                                          struct sdap_options *opts,
                                          struct be_ctx *be,
                                          struct sdap_service *service,
-                                         bool skip_rootdse);
+                                         bool skip_rootdse,
+                                         enum connect_tls force_tls,
+                                         bool skip_auth);
 int sdap_cli_connect_recv(struct tevent_req *req,
                           TALLOC_CTX *memctx,
                           bool *can_retry,
@@ -147,12 +171,13 @@ struct tevent_req *sdap_get_generic_send(TALLOC_CTX *memctx,
                                          const char **attrs,
                                          struct sdap_attr_map *map,
                                          int map_num_attrs,
-                                         int timeout);
+                                         int timeout,
+                                         bool allow_paging);
 int sdap_get_generic_recv(struct tevent_req *req,
                          TALLOC_CTX *mem_ctx, size_t *reply_count,
                          struct sysdb_attrs ***reply_list);
 
-bool sdap_has_deref_support(struct sdap_handle *sh);
+bool sdap_has_deref_support(struct sdap_handle *sh, struct sdap_options *opts);
 
 struct tevent_req *
 sdap_deref_search_send(TALLOC_CTX *memctx,
@@ -175,4 +200,53 @@ errno_t sdap_check_aliases(struct sysdb_ctx *sysdb,
                            struct sss_domain_info *dom,
                            struct sdap_options *opts,
                            bool steal_memberships);
+
+errno_t
+sdap_attrs_add_ldap_attr(struct sysdb_attrs *ldap_attrs,
+                         const char *attr_name,
+                         const char *attr_desc,
+                         bool multivalued,
+                         const char *name,
+                         struct sysdb_attrs *attrs);
+
+#define sdap_attrs_add_string(ldap_attrs, attr_name, attr_desc, name, attrs) \
+        sdap_attrs_add_ldap_attr(ldap_attrs, attr_name, attr_desc, \
+                                 false, name, attrs)
+
+#define sdap_attrs_add_list(ldap_attrs, attr_name, attr_desc, name, attrs) \
+    sdap_attrs_add_ldap_attr(ldap_attrs, attr_name, attr_desc,   \
+                             true, name, attrs)
+
+errno_t sdap_save_all_names(const char *name,
+                            struct sysdb_attrs *ldap_attrs,
+                            bool lowercase,
+                            struct sysdb_attrs *attrs);
+
+struct tevent_req *
+sdap_get_services_send(TALLOC_CTX *memctx,
+                       struct tevent_context *ev,
+                       struct sss_domain_info *dom,
+                       struct sysdb_ctx *sysdb,
+                       struct sdap_options *opts,
+                       struct sdap_search_base **search_bases,
+                       struct sdap_handle *sh,
+                       const char **attrs,
+                       const char *filter,
+                       int timeout,
+                       bool enumeration);
+errno_t
+sdap_get_services_recv(TALLOC_CTX *mem_ctx,
+                       struct tevent_req *req,
+                       char **usn_value);
+
+struct tevent_req *
+enum_services_send(TALLOC_CTX *memctx,
+                   struct tevent_context *ev,
+                   struct sdap_id_ctx *id_ctx,
+                   struct sdap_id_op *op,
+                   bool purge);
+
+errno_t
+enum_services_recv(struct tevent_req *req);
+
 #endif /* _SDAP_ASYNC_H_ */
