@@ -25,8 +25,9 @@
 #include "util/util.h"
 #include "db/sysdb.h"
 #include "providers/ldap/sdap_async_private.h"
+#include "providers/ldap/ldap_common.h"
 
-static bool is_dn(const char *str)
+bool is_dn(const char *str)
 {
     int ret;
     LDAPDN dn;
@@ -37,17 +38,18 @@ static bool is_dn(const char *str)
 
 static errno_t sdap_save_netgroup(TALLOC_CTX *memctx,
                                   struct sysdb_ctx *ctx,
-                                  struct sdap_options *opts,
                                   struct sss_domain_info *dom,
+                                  struct sdap_options *opts,
                                   struct sysdb_attrs *attrs,
-                                  char **_timestamp)
+                                  char **_timestamp,
+                                  time_t now)
 {
     struct ldb_message_element *el;
     struct sysdb_attrs *netgroup_attrs;
     const char *name = NULL;
     int ret;
     char *timestamp = NULL;
-    size_t c;
+    char **missing = NULL;
 
     ret = sysdb_attrs_get_el(attrs,
                              opts->netgroup_map[SDAP_AT_NETGROUP_NAME].sys_name,
@@ -65,21 +67,12 @@ static errno_t sdap_save_netgroup(TALLOC_CTX *memctx,
         goto fail;
     }
 
-    ret = sysdb_attrs_get_el(attrs, SYSDB_ORIG_DN, &el);
-    if (ret) {
+    ret = sdap_attrs_add_string(attrs, SYSDB_ORIG_DN,
+                                "original DN",
+                                name, netgroup_attrs);
+    if (ret != EOK) {
         goto fail;
     }
-    if (el->num_values == 0) {
-        DEBUG(7, ("Original DN is not available for [%s].\n", name));
-    } else {
-        DEBUG(7, ("Adding original DN [%s] to attributes of [%s].\n",
-                  el->values[0].data, name));
-        ret = sysdb_attrs_add_string(netgroup_attrs, SYSDB_ORIG_DN,
-                                     (const char *)el->values[0].data);
-        if (ret) {
-            goto fail;
-        }
-    }
 
     ret = sysdb_attrs_get_el(attrs,
                          opts->netgroup_map[SDAP_AT_NETGROUP_MODSTAMP].sys_name,
@@ -104,70 +97,49 @@ static errno_t sdap_save_netgroup(TALLOC_CTX *memctx,
         }
     }
 
-    ret = sysdb_attrs_get_el(attrs,
-                           opts->netgroup_map[SDAP_AT_NETGROUP_TRIPLE].sys_name,
-                           &el);
-    if (ret) {
-        goto fail;
-    }
-    if (el->num_values == 0) {
-        DEBUG(7, ("No netgroup triples for netgroup [%s].\n", name));
-    } else {
-        for(c = 0; c < el->num_values; c++) {
-            ret = sysdb_attrs_add_string(netgroup_attrs,
-                           opts->netgroup_map[SDAP_AT_NETGROUP_TRIPLE].sys_name,
-                            (const char*)el->values[c].data);
-            if (ret) {
-                goto fail;
-            }
-        }
-    }
-
-    ret = sysdb_attrs_get_el(attrs,
-                       opts->netgroup_map[SDAP_AT_NETGROUP_MEMBER].sys_name,
-                       &el);
+    ret = sdap_attrs_add_list(attrs,
+                        opts->netgroup_map[SDAP_AT_NETGROUP_TRIPLE].sys_name,
+                        "netgroup triple",
+                        name, netgroup_attrs);
     if (ret != EOK) {
         goto fail;
     }
-    if (el->num_values == 0) {
-        DEBUG(7, ("No original members for netgroup [%s]\n", name));
 
-    } else {
-        DEBUG(7, ("Adding original members to netgroup [%s]\n", name));
-        for(c = 0; c < el->num_values; c++) {
-            ret = sysdb_attrs_add_string(netgroup_attrs,
-                       opts->netgroup_map[SDAP_AT_NETGROUP_MEMBER].sys_name,
-                       (const char*)el->values[c].data);
-            if (ret) {
-                goto fail;
-            }
-        }
-    }
-
-
-    ret = sysdb_attrs_get_el(attrs, SYSDB_NETGROUP_MEMBER, &el);
+    ret = sdap_attrs_add_list(attrs,
+                        opts->netgroup_map[SDAP_AT_NETGROUP_MEMBER].sys_name,
+                        "original members",
+                        name, netgroup_attrs);
     if (ret != EOK) {
         goto fail;
     }
-    if (el->num_values == 0) {
-        DEBUG(7, ("No members for netgroup [%s]\n", name));
 
-    } else {
-        DEBUG(7, ("Adding members to netgroup [%s]\n", name));
-        for(c = 0; c < el->num_values; c++) {
-            ret = sysdb_attrs_add_string(netgroup_attrs, SYSDB_NETGROUP_MEMBER,
-                                         (const char*)el->values[c].data);
-            if (ret) {
-                goto fail;
-            }
-        }
+    ret = sdap_attrs_add_list(attrs, SYSDB_NETGROUP_MEMBER,
+                        "members", name, netgroup_attrs);
+    if (ret != EOK) {
+        goto fail;
     }
 
     DEBUG(6, ("Storing info for netgroup %s\n", name));
 
-    ret = sysdb_add_netgroup(ctx, dom, name, NULL, netgroup_attrs,
-                             dp_opt_get_int(opts->basic,
-                                            SDAP_ENTRY_CACHE_TIMEOUT));
+    ret = sdap_save_all_names(name, attrs, !dom->case_sensitive,
+                              netgroup_attrs);
+    if (ret != EOK) {
+        DEBUG(1, ("Failed to save netgroup names\n"));
+        goto fail;
+    }
+
+    /* Make sure that any attributes we requested from LDAP that we
+     * did not receive are also removed from the sysdb
+     */
+    ret = list_missing_attrs(attrs, opts->netgroup_map, SDAP_OPTS_NETGROUP,
+                             attrs, &missing);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to list missing attributes\n"));
+        goto fail;
+    }
+
+    ret = sysdb_add_netgroup(ctx, name, NULL, netgroup_attrs, missing,
+                             dom->netgroup_timeout, now);
     if (ret) goto fail;
 
     if (_timestamp) {
@@ -181,16 +153,8 @@ fail:
     return ret;
 }
 
-struct dn_item {
-    const char *dn;
-    struct sysdb_attrs *netgroup;
-    char *cn;
-    struct dn_item *next;
-    struct dn_item *prev;
-};
-
-static errno_t update_dn_list(struct dn_item *dn_list, const size_t count,
-                              struct ldb_message **res, bool *all_resolved)
+errno_t update_dn_list(struct dn_item *dn_list, const size_t count,
+                       struct ldb_message **res, bool *all_resolved)
 {
     struct dn_item *dn_item;
     size_t c;
@@ -223,7 +187,9 @@ static errno_t update_dn_list(struct dn_item *dn_list, const size_t count,
             }
         }
 
-        not_resolved = true;
+        if (dn_item->cn == NULL) {
+            not_resolved = true;
+        }
     }
 
     *all_resolved = !not_resolved;
@@ -235,7 +201,6 @@ struct netgr_translate_members_state {
     struct tevent_context *ev;
     struct sdap_options *opts;
     struct sdap_handle *sh;
-    struct sss_domain_info *dom;
     struct sysdb_ctx *sysdb;
 
     struct sysdb_attrs **netgroups;
@@ -280,7 +245,6 @@ struct tevent_req *netgr_translate_members_send(TALLOC_CTX *memctx,
 
     state->ev = ev;
     state->opts = opts;
-    state->dom = dom;
     state->sh = sh;
     state->sysdb = sysdb;
     state->netgroups = netgroups;
@@ -431,6 +395,7 @@ static errno_t netgr_translate_members_ldap_step(struct tevent_req *req)
     struct netgr_translate_members_state *state = tevent_req_data(req,
                                           struct netgr_translate_members_state);
     const char **cn_attr;
+    char *filter = NULL;
     struct tevent_req *subreq;
     int ret;
 
@@ -454,6 +419,15 @@ static errno_t netgr_translate_members_ldap_step(struct tevent_req *req)
         return EOK;
     }
 
+    if (!sss_ldap_dn_in_search_bases(state, state->dn_item->dn,
+                                     state->opts->netgroup_search_bases,
+                                     &filter)) {
+        /* not in search base, skip it */
+        state->dn_idx = state->dn_item->next;
+        DLIST_REMOVE(state->dn_list, state->dn_item);
+        return netgr_translate_members_ldap_step(req);
+    }
+
     cn_attr = talloc_array(state, const char *, 3);
     if (cn_attr == NULL) {
         DEBUG(1, ("talloc_array failed.\n"));
@@ -465,11 +439,12 @@ static errno_t netgr_translate_members_ldap_step(struct tevent_req *req)
 
     DEBUG(9, ("LDAP base search for [%s].\n", state->dn_item->dn));
     subreq = sdap_get_generic_send(state, state->ev, state->opts, state->sh,
-                                   state->dn_item->dn, LDAP_SCOPE_BASE, NULL,
+                                   state->dn_item->dn, LDAP_SCOPE_BASE, filter,
                                    cn_attr, state->opts->netgroup_map,
                                    SDAP_OPTS_NETGROUP,
                                    dp_opt_get_int(state->opts->basic,
-                                                  SDAP_SEARCH_TIMEOUT));
+                                                  SDAP_SEARCH_TIMEOUT),
+                                   false);
     if (!subreq) {
         DEBUG(1, ("sdap_get_generic_send failed.\n"));
         return ENOMEM;
@@ -567,13 +542,19 @@ struct sdap_get_netgroups_state {
     struct sss_domain_info *dom;
     struct sysdb_ctx *sysdb;
     const char **attrs;
-    const char *filter;
+    const char *base_filter;
+    char *filter;
+    int timeout;
 
     char *higher_timestamp;
     struct sysdb_attrs **netgroups;
     size_t count;
+
+    size_t base_iter;
+    struct sdap_search_base **search_bases;
 };
 
+static errno_t sdap_get_netgroups_next_base(struct tevent_req *req);
 static void sdap_get_netgroups_process(struct tevent_req *subreq);
 static void netgr_translate_members_done(struct tevent_req *subreq);
 
@@ -582,12 +563,14 @@ struct tevent_req *sdap_get_netgroups_send(TALLOC_CTX *memctx,
                                            struct sss_domain_info *dom,
                                            struct sysdb_ctx *sysdb,
                                            struct sdap_options *opts,
+                                           struct sdap_search_base **search_bases,
                                            struct sdap_handle *sh,
                                            const char **attrs,
                                            const char *filter,
                                            int timeout)
 {
-    struct tevent_req *req, *subreq;
+    errno_t ret;
+    struct tevent_req *req;
     struct sdap_get_netgroups_state *state;
 
     req = tevent_req_create(memctx, &state, struct sdap_get_netgroups_state);
@@ -598,26 +581,66 @@ struct tevent_req *sdap_get_netgroups_send(TALLOC_CTX *memctx,
     state->dom = dom;
     state->sh = sh;
     state->sysdb = sysdb;
-    state->filter = filter;
     state->attrs = attrs;
     state->higher_timestamp = NULL;
     state->netgroups =  NULL;
     state->count = 0;
+    state->timeout = timeout;
+    state->base_filter = filter;
+    state->base_iter = 0;
+    state->search_bases = search_bases;
 
-    subreq = sdap_get_generic_send(state, state->ev, state->opts, state->sh,
-                                   dp_opt_get_string(state->opts->basic,
-                                                     SDAP_NETGROUP_SEARCH_BASE),
-                                   LDAP_SCOPE_SUBTREE,
-                                   state->filter, state->attrs,
-                                   state->opts->netgroup_map,
-                                   SDAP_OPTS_NETGROUP, timeout);
+    if (!state->search_bases) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              ("Netgroup lookup request without a netgroup search base\n"));
+        ret = EINVAL;
+        goto done;
+    }
+
+
+    ret = sdap_get_netgroups_next_base(req);
+
+done:
+    if (ret != EOK) {
+        tevent_req_error(req, ret);
+        tevent_req_post(req, state->ev);
+    }
+    return req;
+}
+
+static errno_t sdap_get_netgroups_next_base(struct tevent_req *req)
+{
+    struct tevent_req *subreq;
+    struct sdap_get_netgroups_state *state;
+
+    state = tevent_req_data(req, struct sdap_get_netgroups_state);
+
+    talloc_zfree(state->filter);
+    state->filter = sdap_get_id_specific_filter(state,
+                        state->base_filter,
+                        state->search_bases[state->base_iter]->filter);
+    if (!state->filter) {
+        return ENOMEM;
+    }
+
+    DEBUG(SSSDBG_TRACE_FUNC,
+          ("Searching for netgroups with base [%s]\n",
+           state->search_bases[state->base_iter]->basedn));
+
+    subreq = sdap_get_generic_send(
+            state, state->ev, state->opts, state->sh,
+            state->search_bases[state->base_iter]->basedn,
+            state->search_bases[state->base_iter]->scope,
+            state->filter, state->attrs,
+            state->opts->netgroup_map, SDAP_OPTS_NETGROUP,
+            state->timeout,
+            false);
     if (!subreq) {
-        talloc_zfree(req);
-        return NULL;
+        return ENOMEM;
     }
     tevent_req_set_callback(subreq, sdap_get_netgroups_process, req);
 
-    return req;
+    return EOK;
 }
 
 static void sdap_get_netgroups_process(struct tevent_req *subreq)
@@ -639,6 +662,17 @@ static void sdap_get_netgroups_process(struct tevent_req *subreq)
     DEBUG(6, ("Search for netgroups, returned %d results.\n", state->count));
 
     if (state->count == 0) {
+        /* No netgroups found in this search */
+        state->base_iter++;
+        if (state->search_bases[state->base_iter]) {
+            /* There are more search bases to try */
+            ret = sdap_get_netgroups_next_base(req);
+            if (ret != EOK) {
+                tevent_req_error(req, ENOENT);
+            }
+            return;
+        }
+
         tevent_req_error(req, ENOENT);
         return;
     }
@@ -664,6 +698,7 @@ static void netgr_translate_members_done(struct tevent_req *subreq)
                                                struct sdap_get_netgroups_state);
     int ret;
     size_t c;
+    time_t now;
 
     ret = netgroup_translate_ldap_members_recv(subreq, state, &state->count,
                                                &state->netgroups);
@@ -673,11 +708,14 @@ static void netgr_translate_members_done(struct tevent_req *subreq)
         return;
     }
 
+    now = time(NULL);
     for (c = 0; c < state->count; c++) {
         ret = sdap_save_netgroup(state, state->sysdb,
-                                 state->opts, state->dom,
+                                 state->dom,
+                                 state->opts,
                                  state->netgroups[c],
-                                 &state->higher_timestamp);
+                                 &state->higher_timestamp,
+                                 now);
         if (ret) {
             DEBUG(2, ("Failed to store netgroups.\n"));
             tevent_req_error(req, ret);
