@@ -109,6 +109,10 @@ int sssm_proxy_id_init(struct be_ctx *bectx,
         goto done;
     }
 
+    ret = confdb_get_bool(bectx->cdb, ctx, bectx->conf_path,
+                          CONFDB_PROXY_FAST_ALIAS, false, &ctx->fast_alias);
+    if (ret != EOK) goto done;
+
     libpath = talloc_asprintf(ctx, "libnss_%s.so.2", libname);
     if (!libpath) {
         ret = ENOMEM;
