@@ -30,6 +30,7 @@
 #include <ares.h>
 
 #include "config.h"
+#include "confdb/confdb.h"
 
 #ifndef HAVE_ARES_DATA
 #include "resolv/ares/ares_parse_srv_reply.h"
@@ -41,6 +42,10 @@
 #define RESOLV_DEFAULT_TTL 7200
 #endif  /* RESOLV_DEFAULT_TTL */
 
+#define RESOLV_DEFAULT_TIMEOUT  5
+
+#include "util/util.h"
+
 /*
  * An opaque structure which holds context for a module using the async
  * resolver. Is should be used as a "local-global" variable - in sssd,
@@ -80,6 +85,11 @@ enum restrict_family {
     IPV6_FIRST
 };
 
+/* Read and validate the family order from conf_path in confdb */
+errno_t
+resolv_get_family_order(struct confdb_ctx *cdb, const char *conf_path,
+                        enum restrict_family *family_order);
+
 /* If resolv_hostent->family is AF_INET, then ipaddr points to
  * struct in_addr, else if family is AF_INET6, ipaddr points to
  * struct in6_addr
@@ -112,7 +122,12 @@ int resolv_gethostbyname_recv(struct tevent_req *req, TALLOC_CTX *mem_ctx,
                               struct resolv_hostent **rhostent);
 
 char *
-resolv_get_string_address(TALLOC_CTX *mem_ctx, struct resolv_hostent *hostent);
+resolv_get_string_address_index(TALLOC_CTX *mem_ctx,
+                                struct resolv_hostent *hostent,
+                                unsigned int addrindex);
+
+#define resolv_get_string_address(mem_ctx, hostent) \
+        resolv_get_string_address_index(mem_ctx, hostent, 0)
 
 struct sockaddr_storage *
 resolv_get_sockaddr_address(TALLOC_CTX *mem_ctx, struct resolv_hostent *hostent,
