@@ -91,7 +91,7 @@ autofs_dp_reconnect_init(struct sbus_connection *conn,
                                 "autofs");
         /* all fine */
         if (ret == EOK) {
-            handle_requests_after_reconnect();
+            handle_requests_after_reconnect(be_conn->rctx);
             return;
         }
     }
