@@ -45,14 +45,6 @@ struct dom_ctx {
     struct ldb_result *res;
 };
 
-struct setent_req_list {
-    struct setent_req_list *prev;
-    struct setent_req_list *next;
-    struct getent_ctx *getent_ctx;
-
-    struct tevent_req *req;
-};
-
 struct getent_ctx {
     struct dom_ctx *doms;
     int num;
@@ -78,6 +70,9 @@ struct nss_dom_ctx {
 
     /* Netgroup-specific */
     struct getent_ctx *netgr;
+
+    /* Service-specific */
+    const char *protocol;
 };
 
 struct setent_step_ctx {
@@ -85,6 +80,7 @@ struct setent_step_ctx {
     struct nss_dom_ctx *dctx;
     struct getent_ctx *getent_ctx;
     struct resp_ctx *rctx;
+    struct cli_ctx *cctx;
     bool check_next;
 
     bool returned_to_mainloop;
@@ -105,20 +101,15 @@ struct setent_step_ctx {
     return ret; \
 } while(0)
 
-/* Find a particular domain object by name */
-struct sss_domain_info *nss_get_dom(struct sss_domain_info *doms,
-                                    const char *domain);
-
 /* Finish the request */
 int nss_cmd_done(struct nss_cmd_ctx *cmdctx, int ret);
 
-/* Respond with no entries */
-int fill_empty(struct sss_packet *packet);
+errno_t nss_setent_add_ref(TALLOC_CTX *memctx,
+                           struct getent_ctx *getent_ctx,
+                           struct tevent_req *req);
 
-int setent_remove_ref(TALLOC_CTX *ctx);
-errno_t setent_add_ref(TALLOC_CTX *memctx,
-                       struct getent_ctx *getent_ctx,
-                       struct tevent_req *req);
+void nss_setent_notify_error(struct getent_ctx *getent_ctx, errno_t ret);
+void nss_setent_notify_done(struct getent_ctx *getent_ctx);
 
 errno_t check_cache(struct nss_dom_ctx *dctx,
                     struct nss_ctx *nctx,
