@@ -86,9 +86,7 @@ static void do_successful_login(struct LOCAL_request *lreq)
     NEQ_CHECK_OR_JUMP(ret, EOK, ("sysdb_attrs_add_long failed.\n"),
                       lreq->error, ret, done);
 
-    ret = sysdb_set_user_attr(lreq, lreq->dbctx,
-                              lreq->preq->domain,
-                              lreq->preq->pd->user,
+    ret = sysdb_set_user_attr(lreq->dbctx, lreq->preq->pd->user,
                               lreq->mod_attrs, SYSDB_MOD_REP);
     NEQ_CHECK_OR_JUMP(ret, EOK, ("sysdb_set_user_attr failed.\n"),
                       lreq->error, ret, done);
@@ -128,9 +126,7 @@ static void do_failed_login(struct LOCAL_request *lreq)
     NEQ_CHECK_OR_JUMP(ret, EOK, ("sysdb_attrs_add_long failed.\n"),
                       lreq->error, ret, done);
 
-    ret = sysdb_set_user_attr(lreq, lreq->dbctx,
-                              lreq->preq->domain,
-                              lreq->preq->pd->user,
+    ret = sysdb_set_user_attr(lreq->dbctx, lreq->preq->pd->user,
                               lreq->mod_attrs, SYSDB_MOD_REP);
     NEQ_CHECK_OR_JUMP(ret, EOK, ("sysdb_set_user_attr failed.\n"),
                       lreq->error, ret, done);
@@ -202,9 +198,7 @@ static void do_pam_chauthtok(struct LOCAL_request *lreq)
     NEQ_CHECK_OR_JUMP(ret, EOK, ("sysdb_attrs_add_long failed.\n"),
                       lreq->error, ret, done);
 
-    ret = sysdb_set_user_attr(lreq, lreq->dbctx,
-                              lreq->preq->domain,
-                              lreq->preq->pd->user,
+    ret = sysdb_set_user_attr(lreq->dbctx, lreq->preq->pd->user,
                               lreq->mod_attrs, SYSDB_MOD_REP);
     NEQ_CHECK_OR_JUMP(ret, EOK, ("sysdb_set_user_attr failed.\n"),
                       lreq->error, ret, done);
@@ -254,8 +248,7 @@ int LOCAL_pam_handler(struct pam_auth_req *preq)
 
     pd->pam_status = PAM_SUCCESS;
 
-    ret = sysdb_get_user_attr(lreq, lreq->dbctx,
-                              preq->domain, preq->pd->user,
+    ret = sysdb_get_user_attr(lreq, lreq->dbctx, preq->pd->user,
                               attrs, &res);
     if (ret != EOK) {
         DEBUG(1, ("sysdb_get_user_attr failed.\n"));
