@@ -44,6 +44,8 @@
 #include "responder/pam/pamsrv.h"
 #include "responder/common/negcache.h"
 
+#define DEFAULT_PAM_FD_LIMIT 8192
+
 #define SSS_PAM_SBUS_SERVICE_VERSION 0x0001
 #define SSS_PAM_SBUS_SERVICE_NAME "pam"
 
@@ -174,6 +176,18 @@ static int pam_process_init(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
+    /* Create table for initgroup lookups */
+    ret = sss_hash_create(pctx, 10, &pctx->id_table);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE,
+              ("Could not create initgroups hash table: [%s]",
+               strerror(ret)));
+        goto done;
+    }
+
+    /* Set up file descriptor limits */
+    responder_set_fd_limit(DEFAULT_PAM_FD_LIMIT);
+
     ret = EOK;
 
 done:
@@ -196,6 +210,9 @@ int main(int argc, const char *argv[])
         POPT_TABLEEND
     };
 
+    /* Set debug level to invalid value so we can deside if -d 0 was used. */
+    debug_level = SSSDBG_INVALID;
+
     pc = poptGetContext(argv[0], argc, argv, long_options, 0);
     while((opt = poptGetNextOpt(pc)) != -1) {
         switch(opt) {
@@ -209,6 +226,8 @@ int main(int argc, const char *argv[])
 
     poptFreeContext(pc);
 
+    CONVERT_AND_SET_DEBUG_LEVEL(debug_level);
+
     /* set up things like debug, signals, daemonization, etc... */
     debug_log_file = "sssd_pam";
 
