@@ -27,6 +27,7 @@
 #include <stdlib.h>
 #include <stdint.h>
 #include <string.h>
+#include <stdbool.h>
 #include "sss_cli.h"
 
 static struct sss_nss_getgrent_data {
@@ -35,8 +36,8 @@ static struct sss_nss_getgrent_data {
     uint8_t *data;
 } sss_nss_getgrent_data;
 
-static void sss_nss_getgrent_data_clean(void) {
-
+static void sss_nss_getgrent_data_clean(void)
+{
     if (sss_nss_getgrent_data.data != NULL) {
         free(sss_nss_getgrent_data.data);
         sss_nss_getgrent_data.data = NULL;
@@ -45,6 +46,126 @@ static void sss_nss_getgrent_data_clean(void) {
     sss_nss_getgrent_data.ptr = 0;
 }
 
+enum sss_nss_gr_type {
+    GETGR_NONE,
+    GETGR_NAME,
+    GETGR_GID
+};
+
+static struct sss_nss_getgr_data {
+    enum sss_nss_gr_type type;
+    union {
+        char *grname;
+        gid_t gid;
+    } id;
+
+    uint8_t *repbuf;
+    size_t replen;
+} sss_nss_getgr_data;
+
+static void sss_nss_getgr_data_clean(bool freebuf)
+{
+    if (sss_nss_getgr_data.type == GETGR_NAME) {
+        free(sss_nss_getgr_data.id.grname);
+    }
+    if (freebuf) {
+        free(sss_nss_getgr_data.repbuf);
+    }
+    memset(&sss_nss_getgr_data, 0, sizeof(struct sss_nss_getgr_data));
+}
+
+static enum nss_status sss_nss_get_getgr_cache(const char *name, gid_t gid,
+                                               enum sss_nss_gr_type type,
+                                               uint8_t **repbuf,
+                                               size_t *replen,
+                                               int *errnop)
+{
+    bool freebuf = true;
+    enum nss_status status;
+    int ret = 0;
+
+    if (sss_nss_getgr_data.type != type) {
+        status = NSS_STATUS_NOTFOUND;
+        goto done;
+    }
+
+    switch (type) {
+    case GETGR_NAME:
+        ret = strcmp(name, sss_nss_getgr_data.id.grname);
+        if (ret != 0) {
+            status = NSS_STATUS_NOTFOUND;
+            goto done;
+        }
+        break;
+    case GETGR_GID:
+        if (sss_nss_getgr_data.id.gid != gid) {
+            status = NSS_STATUS_NOTFOUND;
+            goto done;
+        }
+        break;
+    default:
+        status = NSS_STATUS_TRYAGAIN;
+        ret = EINVAL;
+        goto done;
+    }
+
+    /* ok we have it, remove from cache and pass back to the caller */
+    *repbuf = sss_nss_getgr_data.repbuf;
+    *replen = sss_nss_getgr_data.replen;
+
+    /* prevent _clean() from freeing the buffer */
+    freebuf = false;
+    status = NSS_STATUS_SUCCESS;
+
+done:
+    sss_nss_getgr_data_clean(freebuf);
+    *errnop = ret;
+    return status;
+}
+
+/* this function always takes ownership of repbuf and NULLs it before
+ * returning */
+static void sss_nss_save_getgr_cache(const char *name, gid_t gid,
+                                     enum sss_nss_gr_type type,
+                                     uint8_t **repbuf, size_t replen)
+{
+    int ret = 0;
+
+    sss_nss_getgr_data.type = type;
+    sss_nss_getgr_data.repbuf = *repbuf;
+    sss_nss_getgr_data.replen = replen;
+
+    switch (type) {
+    case GETGR_NAME:
+        if (name == NULL) {
+            ret = EINVAL;
+            goto done;
+        }
+        sss_nss_getgr_data.id.grname = strdup(name);
+        if (!sss_nss_getgr_data.id.grname) {
+            ret = ENOMEM;
+            goto done;
+        }
+        break;
+    case GETGR_GID:
+        if (gid == 0) {
+            ret = EINVAL;
+            goto done;
+        }
+        sss_nss_getgr_data.id.gid = gid;
+        break;
+    default:
+        ret = EINVAL;
+        goto done;
+    }
+
+done:
+    if (ret) {
+        sss_nss_getgr_data_clean(true);
+    }
+    *repbuf = NULL;
+}
+
 /* GETGRNAM Request:
  *
  * 0-X: string with name
@@ -78,8 +199,8 @@ struct sss_nss_gr_rep {
 static int sss_nss_getgr_readrep(struct sss_nss_gr_rep *pr,
                                  uint8_t *buf, size_t *len)
 {
-    size_t i, l, slen, ptmem, pad;
-    ssize_t dlen;
+    errno_t ret;
+    size_t i, l, slen, ptmem, pad, dlen, glen;
     char *sbuf;
     uint32_t mem_num;
     uint32_t c;
@@ -98,36 +219,19 @@ static int sss_nss_getgr_readrep(struct sss_nss_gr_rep *pr,
 
     pr->result->gr_name = &(pr->buffer[0]);
     i = 0;
-    while (slen > i && dlen > 0) {
-        pr->buffer[i] = sbuf[i];
-        if (pr->buffer[i] == '\0') break;
-        i++;
-        dlen--;
-    }
-    if (slen <= i) { /* premature end of buf */
-        return EBADMSG;
-    }
-    if (dlen <= 0) { /* not enough memory */
-        return ERANGE; /* not ENOMEM, ERANGE is what glibc looks for */
-    }
-    i++;
-    dlen--;
+
+    ret = sss_readrep_copy_string(sbuf, &i,
+                                  &slen, &dlen,
+                                  &pr->result->gr_name,
+                                  NULL);
+    if (ret != EOK) return ret;
 
     pr->result->gr_passwd = &(pr->buffer[i]);
-    while (slen > i && dlen > 0) {
-        pr->buffer[i] = sbuf[i];
-        if (pr->buffer[i] == '\0') break;
-        i++;
-        dlen--;
-    }
-    if (slen <= i) { /* premature end of buf */
-        return EBADMSG;
-    }
-    if (dlen <= 0) { /* not enough memory */
-        return ERANGE; /* not ENOMEM, ERANGE is what glibc looks for */
-    }
-    i++;
-    dlen--;
+    ret = sss_readrep_copy_string(sbuf, &i,
+                                  &slen, &dlen,
+                                  &pr->result->gr_passwd,
+                                  NULL);
+    if (ret != EOK) return ret;
 
     /* Make sure pr->buffer[i+pad] is 32 bit aligned */
     pad = 0;
@@ -147,22 +251,13 @@ static int sss_nss_getgr_readrep(struct sss_nss_gr_rep *pr,
 
     for (l = 0; l < mem_num; l++) {
         pr->result->gr_mem[l] = &(pr->buffer[ptmem]);
-        while ((slen > i) && (dlen > 0)) {
-            pr->buffer[ptmem] = sbuf[i];
-            if (pr->buffer[ptmem] == '\0') break;
-            i++;
-            dlen--;
-            ptmem++;
-        }
-        if (slen <= i) { /* premature end of buf */
-            return EBADMSG;
-        }
-        if (dlen <= 0) { /* not enough memory */
-            return ERANGE; /* not ENOMEM, ERANGE is what glibc looks for */
-        }
-        i++;
-        dlen--;
-        ptmem++;
+        ret = sss_readrep_copy_string(sbuf, &i,
+                                      &slen, &dlen,
+                                      &pr->result->gr_mem[l],
+                                      &glen);
+        if (ret != EOK) return ret;
+
+        ptmem += glen + 1;
     }
 
     *len = slen -i;
@@ -254,20 +349,33 @@ enum nss_status _nss_sss_getgrnam_r(const char *name, struct group *result,
     struct sss_cli_req_data rd;
     struct sss_nss_gr_rep grrep;
     uint8_t *repbuf;
-    size_t replen, len;
+    size_t replen, len, name_len;
     enum nss_status nret;
     int ret;
 
     /* Caught once glibc passing in buffer == 0x0 */
-    if (!buffer || !buflen) return ERANGE;
+    if (!buffer || !buflen) {
+        *errnop = ERANGE;
+        return NSS_STATUS_TRYAGAIN;
+    }
 
-    rd.len = strlen(name) + 1;
+    ret = sss_strnlen(name, SSS_NAME_MAX, &name_len);
+    if (ret != 0) {
+        *errnop = EINVAL;
+        return NSS_STATUS_NOTFOUND;
+    }
+
+    rd.len = name_len + 1;
     rd.data = name;
 
     sss_nss_lock();
 
-    nret = sss_nss_make_request(SSS_NSS_GETGRNAM, &rd,
-                                &repbuf, &replen, errnop);
+    nret = sss_nss_get_getgr_cache(name, 0, GETGR_NAME,
+                                   &repbuf, &replen, errnop);
+    if (nret == NSS_STATUS_NOTFOUND) {
+        nret = sss_nss_make_request(SSS_NSS_GETGRNAM, &rd,
+                                    &repbuf, &replen, errnop);
+    }
     if (nret != NSS_STATUS_SUCCESS) {
         goto out;
     }
@@ -293,7 +401,11 @@ enum nss_status _nss_sss_getgrnam_r(const char *name, struct group *result,
 
     len = replen - 8;
     ret = sss_nss_getgr_readrep(&grrep, repbuf+8, &len);
-    free(repbuf);
+    if (ret == ERANGE) {
+        sss_nss_save_getgr_cache(name, 0, GETGR_NAME, &repbuf, replen);
+    } else {
+        free(repbuf);
+    }
     if (ret) {
         *errnop = ret;
         nret = NSS_STATUS_TRYAGAIN;
@@ -327,8 +439,12 @@ enum nss_status _nss_sss_getgrgid_r(gid_t gid, struct group *result,
 
     sss_nss_lock();
 
-    nret = sss_nss_make_request(SSS_NSS_GETGRGID, &rd,
-                                &repbuf, &replen, errnop);
+    nret = sss_nss_get_getgr_cache(NULL, gid, GETGR_GID,
+                                   &repbuf, &replen, errnop);
+    if (nret == NSS_STATUS_NOTFOUND) {
+        nret = sss_nss_make_request(SSS_NSS_GETGRGID, &rd,
+                                    &repbuf, &replen, errnop);
+    }
     if (nret != NSS_STATUS_SUCCESS) {
         goto out;
     }
@@ -354,7 +470,11 @@ enum nss_status _nss_sss_getgrgid_r(gid_t gid, struct group *result,
 
     len = replen - 8;
     ret = sss_nss_getgr_readrep(&grrep, repbuf+8, &len);
-    free(repbuf);
+    if (ret == ERANGE) {
+        sss_nss_save_getgr_cache(NULL, gid, GETGR_GID, &repbuf, replen);
+    } else {
+        free(repbuf);
+    }
     if (ret) {
         *errnop = ret;
         nret = NSS_STATUS_TRYAGAIN;
