@@ -122,6 +122,8 @@ int main(int argc, const char *argv[])
         POPT_TABLEEND
     };
 
+    /* Set debug level to invalid value so we can deside if -d 0 was used. */
+    debug_level = SSSDBG_INVALID;
 
     pc = poptGetContext(argv[0], argc, argv, long_options, 0);
     while((opt = poptGetNextOpt(pc)) != -1) {
@@ -135,7 +137,8 @@ int main(int argc, const char *argv[])
     }
     poptFreeContext(pc);
 
-    debug_level = debug;
+    CONVERT_AND_SET_DEBUG_LEVEL(debug_level);
+
     tests_set_cwd();
 
     Suite *s = crypto_suite();
