@@ -310,6 +310,9 @@ int main(int argc, const char *argv[])
         POPT_TABLEEND
     };
 
+    /* Set debug level to invalid value so we can deside if -d 0 was used. */
+    debug_level = SSSDBG_INVALID;
+
     pc = poptGetContext(argv[0], argc, (const char **) argv, long_options, 0);
     while((opt = poptGetNextOpt(pc)) != -1) {
         fprintf(stderr, "\nInvalid option %s: %s\n\n",
@@ -318,7 +321,8 @@ int main(int argc, const char *argv[])
         return 1;
     }
     poptFreeContext(pc);
-    debug_level = debug;
+
+    CONVERT_AND_SET_DEBUG_LEVEL(debug_level);
 
     tests_set_cwd();
 
