@@ -58,7 +58,7 @@
 #define CHECK_ZERO_ENDPTR(endptr) \
     do { \
         fail_unless(endptr && *endptr == '\0', "Invalid endptr"); \
-    } while(0);
+    } while(0)
 
 /******************
  * strtoint tests *
@@ -434,6 +434,9 @@ int main(int argc, const char *argv[]) {
         POPT_TABLEEND
     };
 
+    /* Set debug level to invalid value so we can deside if -d 0 was used. */
+    debug_level = SSSDBG_INVALID;
+
     pc = poptGetContext(argv[0], argc, argv, long_options, 0);
     while((opt = poptGetNextOpt(pc)) != -1) {
         switch(opt) {
@@ -446,6 +449,8 @@ int main(int argc, const char *argv[]) {
     }
     poptFreeContext(pc);
 
+    CONVERT_AND_SET_DEBUG_LEVEL(debug_level);
+
     tests_set_cwd();
 
     strtonum_suite = create_strtonum_suite();
