@@ -35,7 +35,7 @@
 int main(int argc, const char **argv)
 {
     gid_t pc_gid = 0;
-    int pc_debug = 0;
+    int pc_debug = SSSDBG_DEFAULT;
     struct poptOption long_options[] = {
         POPT_AUTOHELP
         { "debug", '\0', POPT_ARG_INT | POPT_ARGFLAG_DOC_HIDDEN, &pc_debug,
@@ -98,7 +98,7 @@ int main(int argc, const char **argv)
         BAD_POPT_PARAMS(pc, _("Specify group to modify\n"), ret, fini);
     }
 
-    debug_level = pc_debug;
+    debug_level = debug_convert_old_level(pc_debug);
 
     CHECK_ROOT(ret, debug_prg_name);
 
@@ -121,9 +121,7 @@ int main(int argc, const char **argv)
         goto fini;
     }
     /* check the username to be able to give sensible error message */
-    ret = sysdb_getgrnam_sync(tctx, tctx->sysdb,
-                              tctx->octx->name, tctx->local,
-                              tctx->octx);
+    ret = sysdb_getgrnam_sync(tctx, tctx->sysdb, tctx->octx->name, tctx->octx);
     if (ret != EOK) {
         ERROR("Cannot find group in local domain, "
               "modifying groups is allowed only in local domain\n");
