@@ -52,9 +52,7 @@ struct sync_op_res {
 /*
  * Generic modify groups member
  */
-static int mod_groups_member(TALLOC_CTX *mem_ctx,
-                             struct sysdb_ctx *sysdb,
-                             struct sss_domain_info *domain,
+static int mod_groups_member(struct sysdb_ctx *sysdb,
                              char **grouplist,
                              struct ldb_dn *member_dn,
                              int optype)
@@ -63,6 +61,7 @@ static int mod_groups_member(TALLOC_CTX *mem_ctx,
     struct ldb_dn *parent_dn;
     int ret;
     int i;
+    struct sss_domain_info *domain = sysdb_ctx_get_domain(sysdb);
 
     tmpctx = talloc_new(NULL);
     if (!tmpctx) {
@@ -80,8 +79,7 @@ static int mod_groups_member(TALLOC_CTX *mem_ctx,
             goto done;
         }
 
-        ret = sysdb_mod_group_member(tmpctx, sysdb,
-                                     member_dn, parent_dn, optype);
+        ret = sysdb_mod_group_member(sysdb, member_dn, parent_dn, optype);
         if (ret) {
             goto done;
         }
@@ -94,12 +92,12 @@ done:
     return ret;
 }
 
-#define add_to_groups(memctx, sysdb, data, member_dn) \
-    mod_groups_member(memctx, sysdb, data->domain, \
-                      data->addgroups, member_dn, LDB_FLAG_MOD_ADD)
-#define remove_from_groups(memctx, sysdb, data, member_dn) \
-    mod_groups_member(memctx, sysdb, data->domain, \
-                      data->rmgroups, member_dn, LDB_FLAG_MOD_DELETE)
+#define add_to_groups(sysdb, data, member_dn) \
+    mod_groups_member(sysdb, data->addgroups, member_dn, \
+                      LDB_FLAG_MOD_ADD)
+#define remove_from_groups(sysdb, data, member_dn) \
+    mod_groups_member(sysdb, data->rmgroups, member_dn, \
+                      LDB_FLAG_MOD_DELETE)
 
 /*
  * Modify a user
@@ -223,30 +221,28 @@ int usermod(TALLOC_CTX *mem_ctx,
     }
 
     if (attrs->num != 0) {
-        ret = sysdb_set_user_attr(mem_ctx, sysdb,
-                                  data->domain, data->name,
-                                  attrs, SYSDB_MOD_REP);
+        ret = sysdb_set_user_attr(sysdb, data->name, attrs, SYSDB_MOD_REP);
         if (ret) {
             return ret;
         }
     }
 
     if (data->rmgroups != NULL) {
-        ret = remove_from_groups(mem_ctx, sysdb, data, member_dn);
+        ret = remove_from_groups(sysdb, data, member_dn);
         if (ret) {
             return ret;
         }
     }
 
     if (data->addgroups != NULL) {
-        ret = add_to_groups(mem_ctx, sysdb, data, member_dn);
+        ret = add_to_groups(sysdb, data, member_dn);
         if (ret) {
             return ret;
         }
     }
 
-    flush_nscd_cache(mem_ctx, NSCD_DB_PASSWD);
-    flush_nscd_cache(mem_ctx, NSCD_DB_GROUP);
+    flush_nscd_cache(NSCD_DB_PASSWD);
+    flush_nscd_cache(NSCD_DB_GROUP);
 
     return EOK;
 }
@@ -280,29 +276,27 @@ int groupmod(TALLOC_CTX *mem_ctx,
             return ret;
         }
 
-        ret = sysdb_set_group_attr(mem_ctx, sysdb,
-                                   data->domain, data->name,
-                                   attrs, SYSDB_MOD_REP);
+        ret = sysdb_set_group_attr(sysdb, data->name, attrs, SYSDB_MOD_REP);
         if (ret) {
             return ret;
         }
     }
 
     if (data->rmgroups != NULL) {
-        ret = remove_from_groups(mem_ctx, sysdb, data, member_dn);
+        ret = remove_from_groups(sysdb, data, member_dn);
         if (ret) {
             return ret;
         }
     }
 
     if (data->addgroups != NULL) {
-        ret = add_to_groups(mem_ctx, sysdb, data, member_dn);
+        ret = add_to_groups(sysdb, data, member_dn);
         if (ret) {
             return ret;
         }
     }
 
-    flush_nscd_cache(mem_ctx, NSCD_DB_GROUP);
+    flush_nscd_cache(NSCD_DB_GROUP);
 
     return EOK;
 }
@@ -476,9 +470,8 @@ int useradd(TALLOC_CTX *mem_ctx,
 {
     int ret;
 
-    ret = sysdb_add_user(mem_ctx, sysdb,
-                         data->domain, data->name, data->uid, data->gid,
-                         data->gecos, data->home, data->shell, NULL, 0);
+    ret = sysdb_add_user(sysdb, data->name, data->uid, data->gid,
+                         data->gecos, data->home, data->shell, NULL, 0, 0);
     if (ret) {
         goto done;
     }
@@ -493,14 +486,14 @@ int useradd(TALLOC_CTX *mem_ctx,
             goto done;
         }
 
-        ret = add_to_groups(mem_ctx, sysdb, data, member_dn);
+        ret = add_to_groups(sysdb, data, member_dn);
         if (ret) {
             goto done;
         }
     }
 
-    flush_nscd_cache(mem_ctx, NSCD_DB_PASSWD);
-    flush_nscd_cache(mem_ctx, NSCD_DB_GROUP);
+    flush_nscd_cache(NSCD_DB_PASSWD);
+    flush_nscd_cache(NSCD_DB_GROUP);
 
 done:
     return ret;
@@ -528,8 +521,8 @@ int userdel(TALLOC_CTX *mem_ctx,
         DEBUG(2, ("Removing user failed: %s (%d)\n", strerror(ret), ret));
     }
 
-    flush_nscd_cache(mem_ctx, NSCD_DB_PASSWD);
-    flush_nscd_cache(mem_ctx, NSCD_DB_GROUP);
+    flush_nscd_cache(NSCD_DB_PASSWD);
+    flush_nscd_cache(NSCD_DB_GROUP);
 
     return ret;
 }
@@ -537,17 +530,14 @@ int userdel(TALLOC_CTX *mem_ctx,
 /*
  * Public interface for adding groups
  */
-int groupadd(TALLOC_CTX *mem_ctx,
-            struct sysdb_ctx *sysdb,
-            struct ops_ctx *data)
+int groupadd(struct sysdb_ctx *sysdb,
+             struct ops_ctx *data)
 {
     int ret;
 
-    ret = sysdb_add_group(mem_ctx, sysdb,
-                          data->domain, data->name,
-                          data->gid, NULL, 0);
+    ret = sysdb_add_group(sysdb, data->name, data->gid, NULL, 0, 0);
     if (ret == EOK) {
-        flush_nscd_cache(mem_ctx, NSCD_DB_GROUP);
+        flush_nscd_cache(NSCD_DB_GROUP);
     }
     return ret;
 }
@@ -574,7 +564,7 @@ int groupdel(TALLOC_CTX *mem_ctx,
         DEBUG(2, ("Removing group failed: %s (%d)\n", strerror(ret), ret));
     }
 
-    flush_nscd_cache(mem_ctx, NSCD_DB_GROUP);
+    flush_nscd_cache(NSCD_DB_GROUP);
 
     return ret;
 }
@@ -585,14 +575,13 @@ int groupdel(TALLOC_CTX *mem_ctx,
 int sysdb_getpwnam_sync(TALLOC_CTX *mem_ctx,
                         struct sysdb_ctx *sysdb,
                         const char *name,
-                        struct sss_domain_info *domain,
                         struct ops_ctx *out)
 {
     struct ldb_result *res;
     const char *str;
     int ret;
 
-    ret = sysdb_getpwnam(mem_ctx, sysdb, domain, name, &res);
+    ret = sysdb_getpwnam(mem_ctx, sysdb, name, &res);
     if (ret) {
         return ret;
     }
@@ -659,14 +648,13 @@ int sysdb_getpwnam_sync(TALLOC_CTX *mem_ctx,
 int sysdb_getgrnam_sync(TALLOC_CTX *mem_ctx,
                         struct sysdb_ctx *sysdb,
                         const char *name,
-                        struct sss_domain_info *domain,
                         struct ops_ctx *out)
 {
     struct ldb_result *res;
     const char *str;
     int ret;
 
-    ret = sysdb_getgrnam(mem_ctx, sysdb, domain, name, &res);
+    ret = sysdb_getgrnam(mem_ctx, sysdb, name, &res);
     if (ret) {
         return ret;
     }
