@@ -38,7 +38,7 @@ int main(int argc, const char **argv)
     const char *pc_gecos = NULL;
     const char *pc_home = NULL;
     char *pc_shell = NULL;
-    int pc_debug = 0;
+    int pc_debug = SSSDBG_DEFAULT;
     int pc_create_home = 0;
     const char *pc_username = NULL;
     const char *pc_skeldir = NULL;
@@ -96,7 +96,7 @@ int main(int argc, const char **argv)
         }
     }
 
-    debug_level = pc_debug;
+    debug_level = debug_convert_old_level(pc_debug);
 
     if (ret != -1) {
         BAD_POPT_PARAMS(pc, poptStrerror(ret), ret, fini);
@@ -212,7 +212,6 @@ int main(int argc, const char **argv)
             ret = sysdb_getpwnam_sync(tctx,
                                       tctx->sysdb,
                                       tctx->octx->name,
-                                      tctx->local,
                                       tctx->octx);
             if (ret != EOK) {
                 ERROR("Cannot get info about the user\n");
@@ -221,10 +220,8 @@ int main(int argc, const char **argv)
             }
         }
 
-        ret = create_homedir(tctx,
-                             tctx->octx->skeldir,
+        ret = create_homedir(tctx->octx->skeldir,
                              tctx->octx->home,
-                             tctx->octx->name,
                              tctx->octx->uid,
                              tctx->octx->gid,
                              tctx->octx->umask);
