@@ -120,7 +120,7 @@ int main(int argc, const char **argv)
     struct tools_ctx *tctx = NULL;
     const char *pc_username = NULL;
 
-    int pc_debug = 0;
+    int pc_debug = SSSDBG_DEFAULT;
     int pc_remove = 0;
     int pc_force = 0;
     int pc_kick = 0;
@@ -173,7 +173,7 @@ int main(int argc, const char **argv)
         }
     }
 
-    debug_level = pc_debug;
+    debug_level = debug_convert_old_level(pc_debug);
 
     if (ret != -1) {
         BAD_POPT_PARAMS(pc, poptStrerror(ret), ret, fini);
@@ -219,7 +219,6 @@ int main(int argc, const char **argv)
     ret = sysdb_getpwnam_sync(tctx,
                               tctx->sysdb,
                               tctx->octx->name,
-                              tctx->local,
                               tctx->octx);
     if (ret != EOK) {
         /* Error message will be printed in the switch */
