@@ -8,7 +8,8 @@
 /* SHA512-based Unix crypt implementation.
    Released into the Public Domain by Ulrich Drepper <drepper@redhat.com>.  */
 
-#define _GNU_SOURCE
+#include "config.h"
+
 #include <endian.h>
 #include <errno.h>
 #include <limits.h>
