@@ -33,9 +33,9 @@
 
 const char *debug_prg_name = "sssd";
 
-int debug_level = SSS_UNRESOLVED_DEBUG_LEVEL;
-int debug_timestamps = 1;
-
+int debug_level = SSSDBG_UNRESOLVED;
+int debug_timestamps = SSSDBG_TIMESTAMP_UNRESOLVED;
+int debug_microseconds = SSSDBG_MICROSECONDS_UNRESOLVED;
 int debug_to_file = 0;
 const char *debug_log_file = "sssd";
 FILE *debug_file = NULL;
@@ -43,11 +43,14 @@ FILE *debug_file = NULL;
 errno_t set_debug_file_from_fd(const int fd)
 {
     FILE *dummy;
+    errno_t ret;
 
+    errno = 0;
     dummy = fdopen(fd, "a");
     if (dummy == NULL) {
-        DEBUG(1, ("fdopen failed [%d][%s].\n", errno, strerror(errno)));
-        return errno;
+        ret = errno;
+        DEBUG(1, ("fdopen failed [%d][%s].\n", ret, strerror(ret)));
+        return ret;
     }
 
     debug_file = dummy;
@@ -55,6 +58,46 @@ errno_t set_debug_file_from_fd(const int fd)
     return EOK;
 }
 
+int debug_convert_old_level(int old_level)
+{
+    if ((old_level != 0) && !(old_level & 0x000F))
+        return old_level;
+
+    int new_level = SSSDBG_FATAL_FAILURE;
+
+    if (old_level <= 0)
+        return new_level;
+
+    if (old_level >= 1)
+        new_level |= SSSDBG_CRIT_FAILURE;
+
+    if (old_level >= 2)
+        new_level |= SSSDBG_OP_FAILURE;
+
+    if (old_level >= 3)
+        new_level |= SSSDBG_MINOR_FAILURE;
+
+    if (old_level >= 4)
+        new_level |= SSSDBG_CONF_SETTINGS;
+
+    if (old_level >= 5)
+        new_level |= SSSDBG_FUNC_DATA;
+
+    if (old_level >= 6)
+        new_level |= SSSDBG_TRACE_FUNC;
+
+    if (old_level >= 7)
+        new_level |= SSSDBG_TRACE_LIBS;
+
+    if (old_level >= 8)
+        new_level |= SSSDBG_TRACE_INTERNAL;
+
+    if (old_level >= 9)
+        new_level |= SSSDBG_TRACE_ALL;
+
+    return new_level;
+}
+
 void debug_fn(const char *format, ...)
 {
     va_list ap;
@@ -67,25 +110,49 @@ void debug_fn(const char *format, ...)
     va_end(ap);
 }
 
+int debug_get_level(int old_level)
+{
+    if ((old_level != 0) && !(old_level & 0x000F))
+        return old_level;
+
+    if ((old_level > 9) || (old_level < 0))
+        return SSSDBG_FATAL_FAILURE;
+
+    int levels[] = {
+        SSSDBG_FATAL_FAILURE,   /* 0 */
+        SSSDBG_CRIT_FAILURE,
+        SSSDBG_OP_FAILURE,
+        SSSDBG_MINOR_FAILURE,
+        SSSDBG_CONF_SETTINGS,
+        SSSDBG_FUNC_DATA,
+        SSSDBG_TRACE_FUNC,
+        SSSDBG_TRACE_LIBS,
+        SSSDBG_TRACE_INTERNAL,
+        SSSDBG_TRACE_ALL        /* 9 */
+    };
+
+    return levels[old_level];
+}
+
 void ldb_debug_messages(void *context, enum ldb_debug_level level,
                         const char *fmt, va_list ap)
 {
-    int loglevel = -1;
+    int loglevel = SSSDBG_UNRESOLVED;
     int ret;
     char * message = NULL;
 
     switch(level) {
     case LDB_DEBUG_FATAL:
-        loglevel = 0;
+        loglevel = SSSDBG_FATAL_FAILURE;
         break;
     case LDB_DEBUG_ERROR:
-        loglevel = 1;
+        loglevel = SSSDBG_CRIT_FAILURE;
         break;
     case LDB_DEBUG_WARNING:
-        loglevel = 6;
+        loglevel = SSSDBG_TRACE_FUNC;
         break;
     case LDB_DEBUG_TRACE:
-        loglevel = 9;
+        loglevel = SSSDBG_TRACE_ALL;
         break;
     }
 
@@ -95,19 +162,8 @@ void ldb_debug_messages(void *context, enum ldb_debug_level level,
         return;
     }
 
-    if (loglevel <= debug_level) {
-        if (debug_timestamps) {
-            time_t rightnow = time(NULL);
-            char stamp[25];
-            memcpy(stamp, ctime(&rightnow), 24);
-            stamp[24] = '\0';
-            debug_fn("(%s) [%s] [ldb] (%d): %s\n",
-                     stamp, debug_prg_name, loglevel, message);
-        } else {
-            debug_fn("[%s] [ldb] (%d): %s\n",
-                     debug_prg_name, loglevel, message);
-        }
-    }
+    DEBUG_MSG(loglevel, "ldb", message);
+
     free(message);
 }
 
