@@ -117,9 +117,6 @@ int pidfile(const char *path, const char *name)
     fd = open(file, O_RDONLY, 0644);
     err = errno;
     if (fd != -1) {
-
-        pid_str[pidlen] = '\0';
-
         len = 0;
         while ((ret = read(fd, pid_str + len, pidlen - len)) != 0) {
             if (ret == -1) {
@@ -132,7 +129,9 @@ int pidfile(const char *path, const char *name)
                 len += ret;
                 if (len > pidlen) {
                     DEBUG(1, ("read too much, this should never happen.\n"));
-                    break;
+                    close(fd);
+                    talloc_free(file);
+                    return EINVAL;
                 }
                 continue;
             } else {
@@ -141,6 +140,9 @@ int pidfile(const char *path, const char *name)
             }
         }
 
+        /* Ensure NULL-termination */
+        pid_str[len] = '\0';
+
         if (ret == 0) {
             /* let's check the pid */
 
@@ -382,6 +384,7 @@ int server_setup(const char *name, int flags,
     int ret = EOK;
     bool dt;
     bool dl;
+    bool dm;
     struct tevent_signal *tes;
     struct logrotate_ctx *lctx;
 
@@ -459,31 +462,50 @@ int server_setup(const char *name, int flags,
         return ret;
     }
 
-    if (debug_level == SSS_UNRESOLVED_DEBUG_LEVEL) {
+    if (debug_level == SSSDBG_UNRESOLVED) {
         /* set debug level if any in conf_entry */
         ret = confdb_get_int(ctx->confdb_ctx, ctx, conf_entry,
                              CONFDB_SERVICE_DEBUG_LEVEL,
-                             SSS_DEFAULT_DEBUG_LEVEL,
+                             SSSDBG_DEFAULT,
                              &debug_level);
         if (ret != EOK) {
             DEBUG(0, ("Error reading from confdb (%d) [%s]\n",
                       ret, strerror(ret)));
             return ret;
         }
+
+        debug_level = debug_convert_old_level(debug_level);
     }
 
     /* same for debug timestamps */
-    dt = (debug_timestamps != 0);
-    ret = confdb_get_bool(ctx->confdb_ctx, ctx, conf_entry,
-                          CONFDB_SERVICE_DEBUG_TIMESTAMPS,
-                          dt, &dt);
-    if (ret != EOK) {
-        DEBUG(0, ("Error reading from confdb (%d) [%s]\n",
-                  ret, strerror(ret)));
-        return ret;
+    if (debug_timestamps == SSSDBG_TIMESTAMP_UNRESOLVED) {
+        ret = confdb_get_bool(ctx->confdb_ctx, ctx, conf_entry,
+                              CONFDB_SERVICE_DEBUG_TIMESTAMPS,
+                              SSSDBG_TIMESTAMP_DEFAULT,
+                              &dt);
+        if (ret != EOK) {
+            DEBUG(0, ("Error reading from confdb (%d) [%s]\n",
+                      ret, strerror(ret)));
+            return ret;
+        }
+        if (dt) debug_timestamps = 1;
+        else debug_timestamps = 0;
+    }
+
+    /* same for debug microseconds */
+    if (debug_microseconds == SSSDBG_MICROSECONDS_UNRESOLVED) {
+        ret = confdb_get_bool(ctx->confdb_ctx, ctx, conf_entry,
+                              CONFDB_SERVICE_DEBUG_MICROSECONDS,
+                              SSSDBG_MICROSECONDS_DEFAULT,
+                              &dm);
+        if (ret != EOK) {
+            DEBUG(0, ("Error reading from confdb (%d) [%s]\n",
+                      ret, strerror(ret)));
+            return ret;
+        }
+        if (dm) debug_microseconds = 1;
+        else debug_microseconds = 0;
     }
-    if (dt) debug_timestamps = 1;
-    else debug_timestamps = 0;
 
     /* same for debug to file */
     dl = (debug_to_file != 0);
