@@ -20,9 +20,11 @@
 
 #include <ctype.h>
 #include <netdb.h>
+#include <poll.h>
 
 #include "talloc.h"
 #include "util/util.h"
+#include "util/sss_utf8.h"
 #include "dhash.h"
 
 /* split a string into an allocated array of strings.
@@ -133,7 +135,7 @@ int split_on_separator(TALLOC_CTX *mem_ctx, const char *str,
 
     list[l] = NULL; /* terminate list */
 
-    if (size) *size = l + 1;
+    if (size) *size = l;
     *_list = list;
 
     return EOK;
@@ -597,3 +599,52 @@ sss_escape_ip_address(TALLOC_CTX *mem_ctx, int family, const char *addr)
     return family == AF_INET6 ? talloc_asprintf(mem_ctx, "[%s]", addr) :
                                 talloc_strdup(mem_ctx, addr);
 }
+
+/* out->len includes terminating '\0' */
+void to_sized_string(struct sized_string *out, const char *in)
+{
+    out->str = in;
+    if (out->str) {
+        out->len = strlen(out->str) + 1;
+    } else {
+        out->len = 0;
+    }
+}
+
+/* based on code from libssh <http://www.libssh.org> */
+ssize_t sss_atomic_io(int fd, void *buf, size_t n, bool do_read)
+{
+    char *b = buf;
+    size_t pos = 0;
+    ssize_t res;
+    struct pollfd pfd;
+
+    pfd.fd = fd;
+    pfd.events = do_read ? POLLIN : POLLOUT;
+
+    while (n > pos) {
+        if (do_read) {
+            res = read(fd, b + pos, n - pos);
+        } else {
+            res = write(fd, b + pos, n - pos);
+        }
+        switch (res) {
+        case -1:
+            if (errno == EINTR) {
+                continue;
+            }
+            if (errno == EAGAIN || errno == EWOULDBLOCK) {
+                (void) poll(&pfd, 1, -1);
+                continue;
+            }
+            return -1;
+        case 0:
+            errno = EPIPE;
+            return pos;
+        default:
+            pos += (size_t) res;
+        }
+    }
+
+    return pos;
+}
